#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
DetectorMap.py
検出器のマップ表示
"""
import sys
#sys.path.append("c:\\mlf\\1029")
#sys.path.append("c:\\source\\dataacq\\src") 
import wx
import matplotlib
# 先に指定されていなかったら、バックエンドを指定
if not matplotlib.get_backend() == 'WXAgg':
    matplotlib.use('WXAgg')
## [inamura 100726]-->
## for checking version of  matplotlib
vers = matplotlib.__version__.split('.')
matplotlib_verNo = int(vers[0])*1000 + int(vers[1])*10 + int(vers[0])
## <--[inamura 100726]

from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigCanvas
from pylab import *
from matplotlib.colors import LogNorm
from matplotlib.figure import Figure
from matplotlib import cm
#import matplotlib.numerix.ma as ma
try:
    import numpy.ma as ma
except:
    import numpy.core.ma as ma
#import matplotlib.numerix as numpy
from numpy import arange, array, meshgrid,zeros, float64
import threading
import os, time, glob
import cStringIO, zlib
try:
    import scipy.weave as weave
    import platform
    wflag = True and (platform.system() != "Windows")
except:
    wflag = False
import vis.D2Vis as D2Vis
from exp.ChopConfiguration import *
from vis.UtilPlot import *
from vis.FastPlot import *

import math ##[inamura 120115]

import Manyo as mm
import Manyo.Utsusemi as mu

#######################################
#  DMatrixObj
#######################################  
class DMatrixObj(object):
    
    """
    エレメントコンテナマトリックスのハンドリングクラス
    """
        
    #########################################
    def __init__(self, parent, matrix, hTable):
        """
        コンストラクタ
        @param  parent    親クラスのインスタンス
        @param  matrix  　　 エレメントコンテナマトリックス
        @param  hTable　      PSD読み出し用のハッシュテーブル
        @retval 無し
        """ 
        self.matrix = matrix
        self.hTable = hTable
        self.numBank = parent.numBank
        self.numPSD = parent.numPSD
        self.numPixel = parent.numPixel
        self.frame = parent.frame
        
        # データ変換クラスのインスタンスを取得
        self.ce= ConvertEC()
        
        # 無効検出器ID 格納用のリストを準備
        self.masked = []

    #########################################
    def ReadMatrix(self, func):
        """
        エレメントコンテナマトリックスを読み込み、カウントマップを作成
        @param  func　     読み込み終了時のコールバック関数
        @retval 無し
        """
        self.progress = 0
        numRead = len(self.hTable)
        # プログレスバーダイアログ表示
        pdg = wx.ProgressDialog("Loading Matrix","Loading time:",numRead)
        # サブスレッドスタート(データ読み込み)
        th = threading.Thread(target=self._ReadData)
        th.setDaemon(True)
        th.start()
        
        # 進捗表示
        while self.progress < numRead:
            wx.Sleep(1)
            pdg.Update(self.progress)
        pdg.Destroy()
        
        # コールバック関数の実行(マップ表示)
        apply(func)

       
    #########################################
    def _ReadData(self):
        """
        エレメントコンテナマトリックスを読み出す
        @param   matrix  エレメントコンテナマトリックス
        @param  hTable　 PSD読み出し用のハッシュテーブル
        @retval 無し 
        """ 
        # データマップ格納用アレイを準備
        iPSD = self.numBank * (self.numPSD+2) - 2
        self.arData = zeros([self.numPixel, iPSD],float64)
        self.arErr = zeros([self.numPixel, iPSD],float64) ##[inamura 120115]
        # マスクデータを格納
        for i in range(self.numPixel):
            for j in range(iPSD):
                self.arData[i,j] = MASKVALUE
        self.matTOF = []
        self.matINT = []
        self.matERR = []   ##[inamura 120115]
        ##[inamura 100129]-->
        #Inamura 
        ##self.searchId = SearchHeaderChoppers()
        ##self.searchId.SetInputP(self.matrix)
        self.searchId = mm.SearchInHeader(self.matrix)
        # PixelInfoのデータリストを準備
        self.PixelInfo = []
        for i in range(self.numPixel):
            psd_list=[]
            for j in range(iPSD):
                psd_list.append((0.0,0.0,0.0))
            self.PixelInfo.append(psd_list)
             
        ##<--[inamura 100129]
        empty_flag = 0
        #make empty
        ec = self.matrix(1,0)
        xx = array(ec.PutXList())
        yy = ec.PutYList()
        for j in range(len(yy)):
            yy[j]=0
        yy = array(yy)

        emp_psdTOF = []
        emp_psdINT = []
        for i in range(self.numPixel):
            emp_psdTOF.append(xx)
            emp_psdINT.append(yy)
        #--Inamura
        
        # ハッシュテーブルの数だけ繰り返す
        #print "psdID=",
        for xi in range(len(self.hTable)):
            psdID = self.hTable[xi]
            
            #Inamura
            #print "Search id =",psdID
            if psdID >=0:
                #print psdID,
                self.searchId.SearchArray("PSDID",psdID)
                vect_index = self.searchId.PutResultIndex(0)
                retId = -1
                if vect_index.size()!=0:
                    retID = vect_index[0]
                if retID == -1:
                    self.masked.append(psdID)
                    # デテクタデータを書き込み
                    self.matTOF.append(emp_psdTOF) 
                    self.matINT.append(emp_psdINT)
                    self.matERR.append(emp_psdINT) ##[inamura 1201015]
                    #print "empty!!"
                else:
                    # エレメントアレイのポインタを取得
                    ea = self.matrix.PutPointer(retID)
                    # エレメントアレイのヘッダからマスク情報を取得
                    eah = ea.PutHeader()
                    if eah.PutInt4("MASKED"):
                        self.masked.append(psdID)
                        print "Masked = ",psdID

                    # データマップの横軸インデックスを求める
                    psdTOF = []
                    psdINT = []
                    psdERR = []  ##[inamura 120115]
                    # ピクセル数だけ読み出す
                    for i in range(self.numPixel):
                        if not psdID in self.masked:
                            ec = ea.PutPointer(i)
                            try:
                                 ech = ec.PutHeader()
                            except:
                                 print "Cannot load Header of ec(%d) in psdId(%d)" % (i,retID)
                                 raise
                                 
                            tc = ech.PutInt4("TotalCounts")
                            yi = self.numPixel - i - 1
                            self.arData[yi, xi]= tc
                            self.arErr[yi, xi] = math.sqrt(tc) ##[inamura 120115]
                            ##[inamura 100129]-->
                            angP = ech.PutDoubleVector("PixelPolarAngle")
                            angA = ech.PutDoubleVector("PixelAzimAngle")
                            posi = ech.PutDoubleVector("PixelPosition")
                            L2 = sqrt(posi[0]*posi[0]+posi[1]*posi[1]+posi[2]*posi[2])
                            self.PixelInfo[yi][xi] = ( angP[0], angA[0], L2 )
                        # ピクセルデータを取得
                        xx = array(ec.PutXList())
                        yy = array(ec.PutYList())
                        ee = array(ec.PutEList()) ##[inamura 120115]
                        psdTOF.append(xx)
                        psdINT.append(yy) 
                        psdERR.append(ee) ##[inamura 120115]
                     # デテクタデータを書き込み
                    self.matTOF.append(psdTOF) 
                    self.matINT.append(psdINT)
                    self.matERR.append(psdERR) ##[inamura 120115]

            self.progress += 1
            
        print "Masked:", self.masked
        
    #########################################    
    def GetMapData(self):
        """
        全検出単位のカウントマップを返す
        @param  無し        @retval カウントマップデータ(PSD×Pixecl の2次元配列)
        """
        ##[inamura 100621]-->
        #return  self.arData + 0.05
        return  self.arData
        ##<--[inamura 100621]

    #########################################    
    def RemakeMapData(self, tof1, tof2, hTable):
        """
        全検出単位のカウントマップを返す
        @param  tof1  　TOF 範囲始値
        @param  tof2　  TOF 範囲終値
        @param  hTable  PSD読み出し用のハッシュテーブル
        @retval カウントマップデータ(PSD×Pixecl の2次元配列)
        """ 
    
        code =\
         """
         double sum = 0.0;
         int i;

         for(int i=0; i < yy.ubound(0) ;i++) {
             if( tof(i) > tof2) {
                 break;    
                 }
             if( tof(i) >= tof1) {
                 sum += yy(i);
             }
         }
         return_val = sum; 

         """
        
        ##[inamura 120115]-->
        code_err =\
         """
         double sum = 0.0;
         int i;

         for(int i=0; i < ee.ubound(0) ;i++) {
             if( tof(i) > tof2) {
                 break;    
                 }
             if( tof(i) >= tof1) {
                 sum += ee(i)*ee(i);
             }
         }
         return_val = sum; 

         """ ##<--[inamura 120115]
        time0 = time.clock()

        # ハッシュテーブル内の数だけ繰り返す
        for xi in range(len(hTable)):
            psdID = hTable[xi]
            # 有効な検出器か
            if psdID >= 0:
                if self.IsValid(psdID):
                    # ピクセル数だけ読み出す
                    for j in range(self.numPixel):
                        """
                        tof= self.matTOF[psdID][j]
                        yy= self.matINT[psdID][j]
                        ee= self.matERR[psdID][j] ##[inamura 120115]
                        """
                        tof= self.matTOF[xi][j]
                        yy= self.matINT[xi][j]
                        ee= self.matERR[xi][j] ## [inamura 120125]
                        # weave がインポートされているか
                        if wflag:
                            # C言語で積算
                            sum = weave.inline(code, ['tof', 'yy','tof1', 'tof2'], type_converters = weave.converters.blitz)
                            sum_e2 = weave.inline(code_err, ['tof', 'ee','tof1', 'tof2'], type_converters = weave.converters.blitz)##[inamura 120115]
                        else:
                            # python で積算
                            sum = 0.0
                            sum_e2 = 0.0 ##[inamura 120115]
                            for i in range(len(yy)):
                                # 範囲の終わりか
                                if tof[i] > tof2:
                                    break
                                # 範囲内か
                                if tof[i] >= tof1:
                                    sum += yy[i]
                                    sum_e2 += ee[i]*ee[i] ##[inamura 120115]
                        # 新たな値に入れ替える            
                        #Inamura
                        #self.arData[j, xi] = sum
                        self.arData[(self.numPixel-j-1),xi]=sum
                        self.arErr[(self.numPixel-j-1),xi] = math.sqrt( sum_e2 ) ##[inamura 120115]
                        #--Inamura
                # Masked PSD
                else:
                    for j in range(self.numPixel):
                        self.arData[j,xi]= MASKVALUE
                         
            self.npsd = psdID
                
        tsec = time.clock() - time0
        print "Calc Time: %.1f sec" % tsec
        ##[inamura 100621]-->
        #return  self.arData + 0.05
        return  self.arData
        ##<--[inamura 100621]
    
    #########################################    
    def GetPSDArray(self, psd):
        """
        指定されたPSD のデータを返す
        @param  psd  psd No.
        @retval PSD のデータ
        """
        #Inamura
        self.searchId.SearchArray("PSDID",psd)
        vect_index = self.searchId.PutResultIndex(0)
        if vect_index.size()==0:
            return (None,None,None)
        psd = vect_index[0]
        #--Inamura
        # 中心のエレメントコンテナのTOFを取得
        ec0 = self.matrix(psd, self.numPixel/2) 
        tof = ec0.PutX()
        # リビニング後のY データの個数
        znum = len(tof)-1
       
        # リビニング後のカウント数アレイ用に全点 0 のアレイを作成        
        zz =zeros((znum, self.numPixel), float64)
        er =zeros((znum, self.numPixel), float64)
        
        # 1検出器分の強度データを取り出す
        for i in range(self.numPixel):
            # エレメントコンテナ取得
            ec = self.matrix(psd, i)
            # 中央のTOF 値でリビニング 
            ecx = ec.ReBin(tof)
            yy = ecx.PutYList()   
            ee = ecx.PutEList()
            # カウント数アレイとエラー値アレイへ格納 
            for j in range(len(yy)):
                zz[j,i]=yy[j]
                er[j,i]=ee[j]
        
        return (tof,zz, er)
    
   
    #########################################    
    def GetMax(self):
        """
        カウント数の最大値(マスク値を除く)
        @param  無し
        @retval 最大値
        """
        ##[inamura 100621]-->
        #max = 1.0
        max = -1.0E10
        ##<--[inamura 100621]
        for i in range(len(self.arData)):
            for j in range(len(self.arData[0])):
                if self.arData[i,j] < MASKVALUE: 
                    if self.arData[i,j] > max:
                        max = self.arData[i,j]
        return  max

    ##[inamura 100621]-->
    #########################################    
    def GetMin(self):
        """
        カウント数の最小値
        @param  無し
        @retval 最小値
        """ 
        min = 1.0E10
        for i in range(len(self.arData)):
            for j in range(len(self.arData[0])):
                 if self.arData[i,j] < min:
                      min = self.arData[i,j]
        return  min
    ##<--[inamura 100621]
    #########################################    
    def GetTitle(self):
        """
        マトリックスのヘッダーよりランNo.とサンプル名を取得
        @param  無し
        @retval Run No. (文字列)
        """
        runNo = "" 
        sample = ""
        try:
            # マトリックスのヘッダーを取得
            mth = self.matrix.PutHeader()
            # ヘッダーストリングを取得
            runNo = mth.PutString("RUNNUMBER")
        except:
            return  ""

        return  runNo

    #########################################
    def GetTOFRange(self):
        """
        データ読み出し時のTOF範囲を取得
        @param  無し
        @retval ヒストグラムデータ(X, Y, er, ヘッダー、xunit, yunit)
        """ 
        # 中心のエレメントコンテナのTOFを取得
        ec0 = self.matrix(0, self.numPixel/2) 
        tof = ec0.PutX()
        n = len(tof)-1
        return (tof[0], tof[n])
    
    #########################################
    def GetHistogram(self, psdID, pixel):
        """
        PSD-ID からヒストグラムを取得
        @param  psdID PSD No
        @param  pixel Pixel No
        @retval ヒストグラムデータ(X, Y, er, ヘッダー、xunit, yunit)
        """
        #Inamura
        self.searchId.SearchArray("PSDID",psdID)
        vect_index = self.searchId.PutResultIndex(0)
        if vect_index.size()==0:
            return None
        psdID = vect_index[0]
        #--Inamura
        
        # エレメントコンテナ取得
        ec = self.matrix(psdID, pixel)
        # Manyo データからプロッタ用データに変換
        pdat = self.ce.ConvertEC(ec)
        # トレースラベルを追加
        label = "%d-%d" % (psdID, pixel)
        pdat[3]["Label"] = label
        return pdat

    #########################################    
    def IntegrateHistogram(self, lstPSDid):
        """
        指定されたヒストグラムを積算して返す
        ヘッダーの PSD-ID には積算情報を書きこむ
        @param  lstPSDid (ブロックNo.、PSD No.、Pixel No. のタプル　のリスト)
        @retval ヒストグラムデータ()
        """     
        # 領域の中心のPSD と　Pixel を求める
        ps = []
        px = []
        for psdID in lstPSDid:
            if not psdID[0] in ps:
                ps.append(psdID[0])
            if not psdID[1] in px:
                px.append(psdID[1])
        pp = ps[len(ps)/2]
        xx = px[len(px)/2]

        #Inamura
        self.searchId.SearchArray("PSDID",pp)
        vect_index = self.searchId.PutResultIndex(0)
        if vect_index.size()==0:
            return (None,None,None)
        pp = vect_index[0]
        #--Inamura

        # 領域の中心のエレメントコンテナのTOFを取得
        ec0 = self.matrix(pp, xx) 
        tof = ec0.PutX()
        
        # 積算 PSD-ID の文字列を作る
        strSum = ""
        
        # リストに含まれるデータを積算
        n = 0.0

        for psdID in lstPSDid:
             
             #Inamura
            self.searchId.SearchArray("PSDID",psdID[0])
            vect_index = self.searchId.PutResultIndex(0)
            if vect_index.size()==0:
                return (None,None,None)
            psd = vect_index[0]
            # 強度とエラーを取得
#            ec = self.matrix(psdID[0], psdID[1])
            ec = self.matrix(psd,psdID[1])
            #--Inamura
            
            # 領域中心のTOFでリビニング
            ## [inamura 100414]-->
            ## At calulating TOF, Binning must be use!!
            #ecx = ec.ReBin(tof)
            ecx = ec.Binning(tof)
            ## <--[inamura 100414]
            yy = ecx.PutYList()   
            er = ecx.PutEList()
            yy = array(yy)
            er = array(er)
            
            # 最初か
            if strSum == "":
                sumyy = yy
                sumer = er*er
                hist = self.ce.ConvertEC(ec)
            # 2回目以降か
            else:
                # Y値リストとエラーを積算
                sumyy = sumyy + yy
                sumer = sumer + er*er
            
            # psdID の文字列を作成
            strDat = "%d-%d" % (psdID[0], psdID[1])
            # 積算文字列を作成
            if strSum == "":
                strSum = strSum + strDat
            else:
                strSum = strSum + "+" + strDat
            # 積算回数をカウント    
            n += 1.0
            
        # 積算回数で割る        
        sumyy = sumyy / n
#        sumer = sqrt(sumer/n)
        sumer = sqrt(sumer)/n    # 2009.06.15 Minakawa 修正
                    
        # ヘッダー内の Label を、積算文字列に変更
        hist[3]["Label"] = strSum
        hist[1] = sumyy
        hist[2] = sumer
        
        # 積算後のヒストグラムデータに変更   
        return hist        


    #########################################
    def IsValid(self, psd):
        """
        マスクされている検出器かどうかを調べる
        @param   psd  PSD ID   
        @retval True: 有効　or  False: 無効
        """ 
        # マスクPSD のリスト中にあるか
        if psd in self.masked:
            return False
        else:
            return True
    
    
    ##[inamura 120117]-->
    #########################################
    def GetLineFromIntensities(self, psdRange, pixelRange, xAxis ):
         """
         Get Intensity as plot along Pixel direction
         @param psdRange   tupple of minimum and maximum PSD
         @param pixelRange tupple of minimum and maximum PixelNo
         @param xAxis      "PSD" or "PIXEL" to be x-axis
         """
         if not xAxis in ["PSD","PIXEL"]:
             raise 
         
         min_psd = psdRange[0]
         max_psd = psdRange[1]
         min_pix = pixelRange[0]
         max_pix = pixelRange[1]
         
         xi_min = -1
         xi_max = -1
         for i in range(len(self.hTable)):
             if min_psd==self.hTable[i]:
                 xi_min = i
                 break
         for i in range(len(self.hTable)):
             if max_psd==self.hTable[i]:
                 xi_max = i
                 break
         
         if xi_min>xi_max:
             tmp = xi_min
             xi_min = xi_max
             xi_max = tmp
         
         yi_min = self.numPixel - max_pix -1
         yi_max = self.numPixel - min_pix -1

         psd_in = []
         for i in range( xi_min, (xi_max+1) ):
             if self.hTable[i]!=-1:
                 psd_in.append( i )
         
         pixel_in = range( yi_min, (yi_max+1) )
         
         xx = []
         yy = []
         ee = []
                  
         if xAxis=="PSD":
             for i in range( len(psd_in) ):
                 xx.append( self.hTable[ psd_in[i] ] )
                 yy.append(0.0)
                 ee.append(0.0)
             for pix in pixel_in:
                 for i in range(len(psd_in)):
                     yy[i] += self.arData[ pix, psd_in[i] ]
                     ee[i] += self.arErr[ pix, psd_in[i] ]*self.arErr[ pix, psd_in[i] ]
             for i in range( len(xx) ):
                 yy[i] = yy[i]/( double(len(pixel_in)) )
                 ee[i] = math.sqrt(ee[i])/( double(len(pixel_in)) )

         else:
             for i in range( len(pixel_in) ):
                 xx.append( self.numPixel-(pixel_in[i]+1) )
                 yy.append(0.0)
                 ee.append(0.0)
             for psd in psd_in:
                 for i in range( len(pixel_in) ):
                     yy[i] += self.arData[ pixel_in[i], psd ]
                     ee[i] += self.arErr[ pixel_in[i], psd ]*self.arErr[ pixel_in[i], psd ]
             for i in range( len(xx) ):
                 yy[i] = yy[i]/( double(len(psd_in)) )
                 ee[i] = math.sqrt(ee[i])/( double(len(psd_in)) )

         label = "%d-%d,%d-%d(along %s)" % (min_psd, min_pix, max_psd, max_pix, xAxis)
         print "label=",label
         header = {"Label":label }
         
         ret = []
         if xAxis=="PSD":
             ret = [xx,yy,ee,header,"PSDID","Counts"]
         else:
             ret = [xx,yy,ee,header,"PIXEL","Counts"]
             
         return ret

    ##<--[inamura 120117]
    
#######################################
#  DQuasiObj
#######################################  
class DQuasiObj(object):
    
    """
    擬似データのハンドリングクラス
    """
    # X値リスト(デバッグ用ダミーデータ)
    xx = [0, 3,   5,    6,   9,   11,  12,  13,  14,    16,  18, 21,  22,  23,  24, 38, 40]
    # Y値リスト(デバッグ用ダミーデータ)
    yy = [0.0,0.0, 1.0, 5.0, 0.0, 0.0, 100.0,  250.0,  180.0, 90.0, 0.0, 0.0 ,2.0, 14.0,0.0, 0.0] 
    # エラーリスト(デバッグ用ダミーデータ)
    er = er = [1, 1, 1, 2, 1, 1, 6, 7, 5, 4, 2, 0, 2, 3, 2, 1]
        
    #########################################
    def __init__(self, parent, fname, hTable):
        """
        擬似データファイル読み込み、カウントマップを作成
        @param  parent    親クラスのインスタンス
        @param  fname  　擬似データファイル名
        @param  hTable　 PSD読み出し用のハッシュテーブル
        @retval 無し
        """ 
        self.numBank = parent.numBank
        self.numPSD = parent.numPSD
        self.numPixel = parent.numPixel

        # 疑似データファイルを開く
        datFile = open(fname, 'r')

        dataList = []
        dummy = []
        # バンクギャップ用のダミーデータを準備
        for i in range(self.numPixel):
            dummy.append(MASKVALUE)
        
        i = 1
        psd = 0
        for line in datFile:
            try:
                # 1検出器分のデータを読み込んで、実数に変換
                data = map(float, line[:-1].split(','))
            except:
                i += 1
                if i > self.numBank:
                    break
                # バンクギャップ用のダミーデータを作成
                dataList.append(dummy)    
                dataList.append(dummy)
                psd += 2
            else:
                if hTable[psd] >=0:
                    # データを追加
                    data.reverse()
                    dataList.append(data)
                else:
                    dataList.append(dummy)
                psd += 1
                
        # データアレイを準備        
        self.arData =zeros((self.numPixel,(self.numPSD+2)*self.numBank - 2))
        
        # 縦横を入れ替え
        for i in range(self.numPixel):
            for j in range((self.numPSD+2)*self.numBank - 2):
                self.arData[i][j] = dataList[j][i]        
        # 疑似データファイルをクローズ
        datFile.close()               

    #########################################    
    def GetMapData(self):
        """
        全検出単位のカウントマップを返す
        @param  無し
        @retval カウントマップデータ(PSD×Pixecl の2次元配列)
        """ 
        return  self.arData + 0.05

    #########################################    
    def RemakeMapData(self, tof1, tof2, hTable):
        """
        全検出単位のカウントマップを返す
        @param  tof1  　TOF 範囲始値
        @param  tof2　  TOF 範囲終値
        @param  hTable  PSD読み出し用のハッシュテーブル
        @retval カウントマップデータ(PSD×Pixecl の2次元配列)
        """ 
        return  self.arData + 0.05
    
    #########################################    
    def GetPSDArray(self, psd):
        """
        指定されたPSD のデータを返す
        @param  psd  psd No.
        @retval PSD のデータ
        """ 

        # Y データの個数
        tof = arange(5000.0, 15100.0, 100.0)
        znum = len(tof)-1

        # 全点 0 のアレイを作成        
        zz =zeros((znum, self.numPixel), float64)
        er =zeros((znum, self.numPixel), float64)

        # 1検出器分の強度データを取り出す
        for i in range(self.numPixel):
            k = 100
            for j in range(znum):
                while True:
                    if k ==  len(self.arData[0]):
                         break
                    yy = self.arData[i,k]
                    if self.arData[i,k] < MASKVALUE: 
                        break
                    k += 1
                    
                zz[j,i]=yy
                if k ==  len(self.arData[0]):
                    break
                k += 1
        zz[10,10] = -1.5
        return (tof,zz, er)

   
    #########################################    
    def GetMax(self):
        """
        カウント数の最大値(マスク値を除く)
        @param  無し
        @retval 最大値
        """ 
        max = 1.0
        for i in range(len(self.arData)):
            for j in range(len(self.arData[0])):
                if self.arData[i,j] < MASKVALUE: 
                    if self.arData[i,j] > max:
                        max = self.arData[i,j]
        return  max
    
    #########################################    
    def GetTitle(self):
        """
        マトリックスのヘッダーよりランNo.とサンプル名を取得
        @param  無し
        @retval Run No. (文字列)
        """
        return  "AMT0002"

    #########################################
    def GetTOFRange(self):
        """
        データ読み出し時のTOF範囲を取得
        @param  無し
        @retval ヒストグラムデータ(X, Y, er, ヘッダー、xunit, yunit)
        """ 
        n = len(self.xx)-1
        return (self.xx[0], self.xx[n])
    
    #########################################
    def GetHistogram(self, psdID, pixel):
        """
        疑似ヒストグラムデータを返す
        @param  psdID PSD No
        @param  pixel Pixel No
        @retval x, y, er のアレイ、x軸単位、Y軸単位、ヘッダーディクショナリのタプル
        """ 
        # ヘッダーディクショナリ作成
        strPSD = "%d-%d" % (psdID, pixel)
        header = {"Label":strPSD, "RunNo":"AMT00001",}   
        
        return (self.xx,self.yy,self.er,header,"micro-sec","Newtrons")              
   
    #########################################    
    def IntegrateHistogram(self, lstPSDid):
        """
        指定されたヒストグラムを積算して返す
        ヘッダーの PSD-ID には積算情報を書きこむ
        @param  lstPSDid (ブロックNo.、PSD No.、Pixel No. のタプル　のリスト)
        @retval ヒストグラムデータ()
        """     

        # 積算 PSD-ID の文字列を作る
        strSum = ""
        
        # リストに含まれるデータを積算
        for psdID in lstPSDid:
            # psdID の文字列を作成
            strDat = "%d-%d" % (psdID[0], psdID[1])
            # 積算文字列を作成
            if strSum == "":
                strSum = strSum + strDat
            else:
                strSum = strSum + "+" + strDat
                
        header = {"Label":strSum, "RunNo":"AMT00001",}   
        return (self.xx,self.yy,self.er,header,"micro-sec","Newtrons")   


    #########################################
    def IsValid(self, psd):
        """
        マスクされている検出器かどうかを調べる
        @param   psd  PSD ID   
        @retval True: 有効　or  False: 無効
        """ 
        return True

#######################################
#  MapFrame
#######################################  
class MapFrame(object):
    """
    検出器マップ表示画面クラス
    """

    #########################################
    def __init__(self, parent, matrix):
        """
        検出器マップ表示処理のコンストラクタ
        @param  parent 親画面のインスタンス   
        @param  matrix エレメントアレイマトリックス   
        @retval 無し
        """ 
        # 初期化
        self.logFlag = False
        self.dragFlag = False
        self.d2 = None
        # プロッタ番号の初期化
        self.order = 0
        # プロッタリストの準備
        self.plots = []
        # カレントプロッタ
        self.plot = None

        # リソースからメイン画面を取得
        res = ResourceFile()
        self.frame = res.GetFrame(parent, "mapframe")
        # リソースからフレームが取得できなかった
        if self.frame == None:
            return
        # アイコンを設定
        self.frame.SetIcon(Images().GetMainIcon())
        # メニューバーを取得
        self.menuBar= res.GetMenuBar('MainMenuBar')
        self.frame.SetMenuBar(self.menuBar)
        # グラフ描画用のパネルを取得
        panel = res.GetCtrl(self.frame, 'mapPanel')

        # グラフ描画用のフィギュアとキャンバスを作成
        self.fig   = Figure(None, 80)
        self.canvas= FigCanvas(panel, -1, self.fig)
        sizer = wx.BoxSizer(wx.VERTICAL)

        # ツールバー作成
        self.toolbar = ChartToolBar(self,0)
        sizer.Add(self.toolbar, 0, wx.LEFT|wx.TOP|wx.GROW)
        self.toolbar.Show()
        sizer.Add(self.canvas, 1, wx.LEFT | wx.EXPAND)
        # サイザーに入れる
        panel.SetSizer(sizer)
        panel.Fit()
        
        # コンフィギュレーションを取得
        try:
            """
            cnf = ChopConfiguration()
            # 分光器名を取得
            instrument = cnf.PutSpectrometerName()
            # 検出器情報を取得
            self.numBank, self.numPSD, self.numPixel = cnf.PutDetectInfo()
            # PSD 情報(ハッシュテーブル)を取得
            self.hTable = cnf.PutPSDInfo()
            """
            self.numBank = 1
            self.numPSD = matrix.PutSize()
            self.numPixel = matrix(0).PutSize()
            self.hTable = []
            for i in range(self.numPSD):
                 if matrix(i).PutHeaderPointer().PutInt4("MASKED")==1:
                      self.hTable.append(-1)
                 else:
                      self.hTable.append( matrix(i).PutHeaderPointer().PutInt4("PSDID") )
            print "hTable=",self.hTable
            instrument = matrix.PutHeaderPointer().PutString("INSTRUMENT")

        except PlotException, ex:
            PlotMessage(self.frame,ex)
            self.OnClose()
            return
        
        # フレームのタイトルに分光器名を付加
        strT1 = self.frame.GetTitle()
        strT2 = " for %s" % instrument
        self.frame.SetTitle(strT1+strT2)
        
        # イベントハンドラーの設定
        self._SetEventHandler(res)
        
        # メイン画面を表示
        self.frame.Show()
        
        time.sleep(0.1) ##[inamura 100726] for stability on command line
        
        # 引数が文字列であったなら
        if type(matrix) == type(""):
            # 疑似データファイル読み込み
            self.do = DQuasiObj(self, matrix, self.hTable)
            self._DispChart()
        else:
            # エレメントコンテナマトリックスをロード
            self.do = DMatrixObj(self, matrix, self.hTable)
            self.do.ReadMatrix(self._DispChart)

    #########################################
    def _DispChart(self):
        """
        検出器マップチャートを描画する
        @param   無し   
        @retval 無し
        """
        # マップデータの取得   
        self.Z = self.do.GetMapData()
        self.datMax = self.do.GetMax()
        self.zMax = self.datMax
        ##[inamura 100621]-->
        self.datMin = self.do.GetMin()
        if (self.logFlag) and (self.datMin <= 0.0):
             self.zMin = 1.0E-20
        else:
             self.zMin = self.datMin
        
        self.yminTx.SetValue( ("%g" % self.zMin) )
        self.ymaxTx.SetValue( ("%g" % self.zMax) )
        ##<--[inamura 100621]
        ##[inamura 100129]-->
        self.PixelInfo = self.do.PixelInfo
        ##<--[inamura 100129]

        # マップデータからマスクデータ作成
        self.Zm = ma.masked_where(self.Z >= MASKVALUE, self.Z)

        # TOF Range の初期値を設定
        tof1, tof2 = self.do.GetTOFRange()
        # 始値のテキストボックスに値を設定    
        strtof1 = "%d" % tof1
        strtof2 = "%d" % tof2
        self.tofTx1.SetValue(strtof1)
        self.tofTx2.SetValue(strtof2)

        # PSD の最大値を検索
        self.maxPSD = 0
        for pnum in self.hTable:
            if pnum > self.maxPSD:
                self.maxPSD = pnum

        # 検出器マップの描画
        self.PlotChart()
        


    #########################################
    def _SetEventHandler(self, res):
        """
        イベントハンドラーの登録
        @param  res リソースファイルのインスタンス   
        @retval 無し
        """ 
        # プリントボタンのリスナー登録
        IFEvtProp(0).AddListner('print',  self.OnPrint)
        # 保存ボタンのリスナー登録
        IFEvtProp(0).AddListner('save',  self.OnSave)
        
        #ボタンのイベントハンドラ登録
        self.frame.Bind(wx.EVT_BUTTON, self.OnBtRedraw, id=ResourceFile().GetId('btRedraw'))
        self.frame.Bind(wx.EVT_BUTTON, self.OnBt2dPlot, id=ResourceFile().GetId('bt2dPlot'))
        self.frame.Bind(wx.EVT_BUTTON, self.OnBtAppend, id=ResourceFile().GetId('btAppend'))
        self.frame.Bind(wx.EVT_BUTTON, self.OnBtSumm, id=ResourceFile().GetId('btSumm'))
        self.frame.Bind(wx.EVT_BUTTON, self.OnNewPlot, id=ResourceFile().GetId('btNewPlot'))
        ##[inamura 100621]-->
        self.frame.Bind(wx.EVT_BUTTON, self.OnBtRedrawInt, id=ResourceFile().GetId('btIntSet'))
        self.frame.Bind(wx.EVT_BUTTON, self.OnBtRedraw, id=ResourceFile().GetId('btIntClear'))
        ##<--[inamura 100621]
 
        #メニューのイベントハンドラ登録
        self.frame.Bind(wx.EVT_MENU, self.OnPrint, id=ResourceFile().GetId('menu_print'))
        self.frame.Bind(wx.EVT_MENU, self.OnClose, id=ResourceFile().GetId('menu_exit'))
        self.frame.Bind(wx.EVT_MENU, self.OnNewPlot, id=ResourceFile().GetId('menu_plot'))
        self.frame.Bind(wx.EVT_MENU, self.OnLog, id=ResourceFile().GetId('menu_log'))
        
        #クローズイベント登録
        self.frame.Bind(wx.EVT_CLOSE, self.OnClose)
        
        #マウスイベントのリスナ登録
        self.canvas.mpl_connect('motion_notify_event', self.OnMouseMove)
        self.canvas.mpl_connect('button_press_event', self.OnBtDown)
        self.canvas.mpl_connect('button_release_event', self.OnBtUp)
        
        #テキストボックスのコントロール取得
        panel = ResourceFile().GetCtrl(self.frame, 'panel0')
        self.psdTx0 = ResourceFile().GetCtrl(panel, 'psd0')
        self.psdTx1 = ResourceFile().GetCtrl(panel, 'PSD1')
        self.pxTx1 = ResourceFile().GetCtrl(panel, 'pixel1')
        self.psdTx2 = ResourceFile().GetCtrl(panel, 'PSD2')
        self.pxTx2 = ResourceFile().GetCtrl(panel, 'pixel2')
        self.tofTx1 = ResourceFile().GetCtrl(panel, 'tof1')
        self.tofTx2 = ResourceFile().GetCtrl(panel, 'tof2')
        
        #リストのコントロール取得
        self.plist = ResourceFile().GetCtrl(panel, 'plotList')
        self.frame.Bind(wx.EVT_LISTBOX, self.OnListSelect, self.plist)
        
        # 検出器情報のコントロール取得
        pn = ResourceFile().GetCtrl(self.frame,'panel2')
        self.iftxt = ResourceFile().GetCtrl(pn,'infotxt')         
        
        #スライダーのイベントハンドラ登録
        panels = ResourceFile().GetCtrl(self.frame,'panel1')
        # リソースからスライダーのID を取得
        self.slider = ResourceFile().GetCtrl(panels,'slider')
        # スライダーのイベントハンドラーを設定
        self.slider.Bind(wx.EVT_SLIDER , self.OnSlider)

        #[inamura 100621]-->
        self.yminTx = ResourceFile().GetCtrl(pn,'ymin')
        self.ymaxTx = ResourceFile().GetCtrl(pn,'ymax')
        #<--[inamura 100621]
        self.cbXax = ResourceFile().GetCtrl(panel, 'cbXaxisOfPlot') # [inamura 120115]
        
    #########################################
    def PlotChart(self):
        """
        検出器マップチャートを描画する
        @param   無し   
        @retval 無し
        """ 
        # フィギュアをクリア 
        self.fig.clf()
        # タイトルを取得
        runNo = self.do.GetTitle()
        strRunNo = "Run No.: " + runNo

        # グラフのタイトルとして、ランNo.を表示
        self.fig.text(0.08, 0.94, strRunNo)
        ##[inamura 100622]-->
        x0 = float(self.tofTx1.GetValue())
        x1 = float(self.tofTx2.GetValue())
        y0 = float(self.yminTx.GetValue())
        y1 = float(self.ymaxTx.GetValue())
        strInfo1 = "X-axis Range\nIntensity Range"
        strInfo2 = ": %g - %g\n: %g - %g" % (x0,x1,y0,y1)
        self.fig.text(0.20, 0.905, strInfo1)
        self.fig.text(0.32, 0.905, strInfo2)
        ##<--[inamura 100622]

        x = arange(0, (self.numPSD+2)*self.numBank - 1, 1)
        y = arange(0, self.numPixel+1, 1)
        X, Y = meshgrid(x, y)

        # 検出器マップ用のサブプロットを準備
        self.ax = self.fig.add_subplot(111)
        self.ax.set_position([0.08, 0.15, 0.89 ,0.73])
        
        # カラーバー用の座標軸を準備
        axc = self.fig.add_axes([0.68, 0.93, 0.25 ,0.03])
        
        # カーソル表示用のサブプロットを準備(透過モード)
        self.cursol = self.fig.add_subplot(111,frameon=False)
        self.cursol.set_position([0.08, 0.15, 0.89 ,0.73])
        
        # マスクデータの色作成(白色)
        cmap = cm.jet
        cmap.set_bad('w', 1.0)
        
        # ログ表示か
        if self.logFlag:
            # カラーマップをログモードで表示
            pc = self.ax.pcolormesh(X, Y, self.Zm,  cmap=cmap,\
                                    norm=LogNorm(vmin=self.zMin, vmax=self.zMax)) ##[inamura 100621]
                                #norm=LogNorm(vmin=0.11, vmax=self.zMax))
        else:
            # カラーマップをリニアモードで表示
            pc = self.ax.pcolormesh(X, Y, self.Zm, cmap=cmap,\
                                    norm=Normalize(vmin=self.zMin, vmax=self.zMax)) ##[inamura 100621]
                                #norm=Normalize(vmin=0, vmax=self.zMax))
            
        # カラーマップと関連付けて、カラーバーを表示     
        self.fig.colorbar(pc, cax=axc, orientation='horizontal')
        # カラーバーの座標ラベルの文字を小さくする
        xlabels = getp(axc, 'xticklabels')
        setp(xlabels, size=7)
        
        # チャートを整える
        self.ArrangeChart()
        
        # グラフを再表示              
        self.canvas.draw()    
    
    #########################################
    def ArrangeChart(self):
        """
        検出器マップチャートを整える
        @param   無し   
        @retval 無し
        """ 
        # X軸範囲とY軸範囲をセット
        self.ax.set_xlim((0.0, (self.numPSD+2)*self.numBank - 2))
        self.ax.set_ylim((0.0, self.numPixel))
        self.cursol.set_xlim((0.0, (self.numPSD+2)*self.numBank - 2))
        self.cursol.set_ylim((0.0, self.numPixel))
        
        # 検出器マップの座標を消す
        self.ax.set_xticks([])
        self.ax.set_yticks([])

        # カーソル用サブプロットの座標を消す
        self.cursol.set_xticks([])
        self.cursol.set_yticks([])

        # ピクセル番号をグラフ左に表示
        ##[inamura 100622]-->
        #hf = self.numPixel / 2
        #strHf = "%d" % hf
        #strPx = "%d" % self.numPixel
        hf = self.numPixel / 2 -1
        strHf = "%d" % hf
        strPx = "%d" % (self.numPixel -1)
        # バンク番号をグラフ下に表示
        #self.ax.text(-3, self.numPixel-1, "1", horizontalalignment='right')
        self.ax.text(-3, self.numPixel-1, "0", horizontalalignment='right')
        self.ax.text(-3, hf, strHf, horizontalalignment='right')
        self.ax.text(-3, 0, strPx, horizontalalignment='right')
        ##<--[inamura 100622]
        self.fig.text(0.5, 0.07, "Bank", horizontalalignment='center')
        
        xx = 0.09
        xP = self.numPSD
        for i in range(self.numBank):
            # バンク No. 表示
            num = i+1.0
            strBk = "%d" % num
            bx = i*(self.numPSD+2) + self.numPSD / 2 + 1.0 
            self.ax.text(bx, -7, strBk, horizontalalignment='center')
            xx += 0.084
            # バンク境界表示
            if not num == self.numBank:
                #self.ax.axvspan(xP, xP+2, facecolor='0.4')
                self.ax.axvspan(xP-0.2, xP+2, facecolor='0.4') ## [inamura 100726] for matplotlib 0.98
                xP = xP + 2 + self.numPSD

    #########################################        
    def UpdateStatusBar(self, event):
        """
        マウス位置の検出器情報を表示
        @param   event イベント情報   
        @retval 無し
        """ 
        # マウス位置を取得
        x, y = int(event.xdata), int(event.ydata)
        # マウス位置の PSD-ID を取得
        #[inamura091012]--
        #pixel = self.numPixel - y
        pixel = self.numPixel - (y+1)
        #--[inamura091012]
        psd = self.hTable[x] 
        # マウス位置の カウント数を取得 
        counts = self.Z[y,x]
        # 検出器があるか
        if self.hTable[x] >= 0:
            strSTDID = "PSD ID: %03d -%03d \n" % (psd, pixel)
            # 有効な検出器か 
            if self.do.IsValid(psd):
                ##[inamura 100129]-->
                #strCounts = "     Counts: %d" % counts
                strCounts = "     2theta:%+08.3f  Azim:%+08.3f  L2:%08.2f\n" % self.PixelInfo[y][x]
                strCounts+= "     Counts: %d" % counts
                ##<--[inamura 100129]
            else:
                ##[inamura 100129]-->
                #strCounts = "     Counts: --"
                strCounts = "     2theta:xxxx.xxx  Azim:xxxx.xxx  L2:xxxxx.xx\n"
                strCounts+= "     Counts: xx\n"
                ##<--[inamura 100129]
        # 無効な検出器の場合は情報を表示しない
        else:
            ##[inamura 100129]-->
            #strSTDID = "        PSD ID: ---- "
            #strCounts = "     Counts: --"
            strSTDID = "PSD ID: xxx -xxx \n"
            strCounts = "     2theta:xxxx.xxx  Azim:xxxx.xxx  L2:xxxxx.xx\n"
            strCounts+= "     Counts: xx"
            ##<--[inamura 100129]
        # マップの下にマウス位置の情報を表示    
        self.iftxt.SetLabel(strSTDID +strCounts)

    #########################################
    def OnMouseMove(self, event):
        """
        マウス-ボタンクリックイベント処理
        @param   event イベント情報   
        @retval 無し
        """ 
        # マウス位置がグラフ内かどうかをチェック
        if not event.inaxes:
            return
        # グラフ内であっても、None が来ることがあるのでプロテクト
        if event.ydata == None or event.xdata == None:
            return
                
        # ドラッグ中であれば
        if self.dragFlag:
            # カーソル用サブプロットをクリア
            self.cursol.clear()
            ##[inamura 100726]-->
            ## if version of matplotlib > 0.98
            if matplotlib_verNo > 980:
                 self.cursol.autoscale(False)
            ##<--[inamura 100726]
            # 現在のY　レンジを取得
            y0, y1 = self.cursol.set_ylim()
            
            if event.ydata-y0 > self.ydata:
                yy0 = self.ydata
                yy1 = event.ydata
            else:
                yy0 = event.ydata
                yy1 = self.ydata
                
            if event.xdata-y0 > self.xdata:
                xx0 = self.xdata
                xx1 = event.xdata
            else:
                xx0 = event.xdata
                xx1 = self.xdata    
            # Y の比率を計算    
            ymin = (yy0-y0)/ (y1-y0)
            ymax = (yy1-y0)/ (y1-y0)

            self.cursol.axvspan(xx0, xx1, ymin=ymin, ymax=ymax, facecolor='w', alpha=0.4)
            # カーソル用サブプロットの座標を消す
            self.cursol.set_xticks([])
            self.cursol.set_yticks([])
            # グラフを再表示              
            self.canvas.draw()
            
        # ドラッグ中でなければ    
        else:
            # マウス位置の検出器ヘッダー情報を表示
            self.UpdateStatusBar(event)         
    
    #########################################
    def OnBtDown(self, event):
        """
        マウス-ボタンクリックイベント処理
        @param   event イベント情報   
        @retval 無し
        """ 
       
        # パン処理中またはズーム処理中であれば、以下の処理を実行しない
        if self.toolbar._active == 'PAN' or self.toolbar._active == 'ZOOM':
            return
        if event.inaxes:
            # ドラッグ開始フラグをセット
            self.dragFlag = True
            # ドラッグ時のカーソル表示用に、開始位置を保存
            self.xdata = event.xdata
            self.ydata = event.ydata
            # クリック位置のPSD-ID を保存
            self.x0, self.y0 = int(event.xdata), int(event.ydata)
        else:
            self.x0 = -1

                
    #########################################
    def OnBtUp(self, event):
        """
        マウス-ボタンアップイベント処理
        @param   event イベント情報   
        @retval 無し
        """ 
        
        # パン処理中またはズーム処理中であれば、以下の処理を実行しない
        if self.toolbar._active == 'PAN' or self.toolbar._active == 'ZOOM':
            return
        # ドラッグ開始時にグラフ外だったか
        if not event.inaxes or self.x0 < 0:
            return
        
        # ドラッグ中であれば
        if self.dragFlag:
            # ドラッグフラグを解除
            self.dragFlag = False
            # ドラッグカーソルを消す
            #self.cursol.clear()             ##[inamura 100726]
            # カーソル用サブプロットの座標を消す
            self.cursol.set_xticks([])
            self.cursol.set_yticks([])
            # グラフを再表示              
            self.canvas.draw()   

        # ボタンアップ時の位置を取得
        x2, y2 = int(event.xdata), int(event.ydata)
        # 始値より小さければ入れ替える
        if self.x0 > x2:
            x1 = x2
            x2 = self.x0
        else:
            x1 = self.x0
            
        if self.y0 < y2:
            y1 = y2
            y2 = self.y0
        else:
            y1 = self.y0    

        #Inamura
        #pixel1 = self.numPixel - y1
        #pixel2= self.numPixel - y2
        pixel1 = self.numPixel - y1 - 1
        pixel2= self.numPixel - y2 - 1
        #--Inamura  
        
        xx = x1
        psi1 = -1
        psi2 = -1
        
        while xx <= x2 :
            # 最も近い PSD 
            if self.hTable[xx] >= 0 and self.do.IsValid(self.hTable[xx]):
                if psi1 < 0:
                    psi1 = self.hTable[xx] 
                    psi2 = self.hTable[xx] 
                else:
                    psi2 = self.hTable[xx]
            xx += 1
        # データ無しか
        if psi1 < 0 :
            self.psdTx0.SetValue("")
            self.psdTx1.SetValue("")
            self.pxTx1.SetValue("")
            self.psdTx2.SetValue("")
            self.pxTx2.SetValue("")
            return

        # 始値のテキストボックスに値を設定    
        strpsd = "%d" % psi1
        strpx = "%d" % pixel1
        self.psdTx0.SetValue(strpsd)
        self.psdTx1.SetValue(strpsd)
        self.pxTx1.SetValue(strpx)
        # ボタン押下時と位置が同じであるなら、ドラッグではなくクリック
        if x1 == x2 and y1 == y2:
            # 終値のテキストボックスをクリア            
            self.psdTx2.SetValue("")
            self.pxTx2.SetValue("")
            ##[inamura 100726]-->
            self.cursol.clear()
            if matplotlib_verNo > 980:
                 self.cursol.autoscale(False)
            self.cursol.set_xticks([])
            self.cursol.set_yticks([])
            self.canvas.draw()
            ##<--[inamura 100726]
        else:
            # 終値のテキストボックスに値を設定
            strpsd = "%d" % psi2
            strpx = "%d" % pixel2
            self.psdTx2.SetValue(strpsd)
            self.pxTx2.SetValue(strpx)
        
    

    #########################################
    def OnBtAppend(self, event):
        """
        Appendボタン押下イベント処理
        @param   event イベント情報   
        @retval 無し
        """ 
        # 個別にデータをプロッタへ追加
        self.AddToPlot(False)

    #########################################
    def OnBtRedraw(self, event):
        """
        指定されたTOFレンジからカウント数を
        再計算し、マップを再表示
        @param   event イベント情報   
        @retval 無し
        """ 
        try:
            try:
                # TOF Range をテキストボックスから取得
                tof1 = float(self.tofTx1.GetValue())
                tof2 = float(self.tofTx2.GetValue())
            except:
                # エラーメッセージを表示   
                raise PlotException('Common','C027',("TOF Range",))
            if tof1 > tof2:
                  raise PlotException('Common','C010',("TOF Range End","TOF Range First"))
            ##[inamura 091114]->
            #if tof1 < 0.0:
            #      raise PlotException('Common','C009',("TOF Range First","0.0"))
            ##<-[inamura 091114]
        except PlotException, ex:
            PlotMessage(self.frame,ex)
            return
        # カーソルを砂時計に変更    
        self.frame.SetCursor(wx.StockCursor(wx.CURSOR_WAIT))    
        # マップデータを再作成    
        self.Z = self.do.RemakeMapData(tof1, tof2, self.hTable)
        # カーソルを戻す   
        self.frame.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))
        
        self.datMax = self.do.GetMax()
        self.zMax = self.datMax
        ##[inamura 100621]-->
        self.datMin = self.do.GetMin()
        if (self.logFlag) and (self.datMin <= 0.0):
             self.zMin = 1.0E-20
        else:
             self.zMin = self.datMin
        
        self.yminTx.SetValue( ("%g" % self.zMin) )
        self.ymaxTx.SetValue( ("%g" % self.zMax) )
        ##<--[inamura 100621]
        
        # マップデータからマスクデータ作成
        self.Zm = ma.masked_where(self.Z >= MASKVALUE, self.Z)
        
        # マップを再描画
        self.PlotChart()    

    ##[inamura 100621]-->
    #########################################
    def OnBtRedrawInt(self, event):
        """
        指定されたIntensityレンジとTOFレンジからカウント数を
        再計算し、マップを再表示
        @param   event イベント情報   
        @retval 無し
        """ 
        try:
            try:
                # TOF Range をテキストボックスから取得
                tof1 = float(self.tofTx1.GetValue())
                tof2 = float(self.tofTx2.GetValue())
                int1 = float(self.yminTx.GetValue())
                int2 = float(self.ymaxTx.GetValue())
            except:
                # エラーメッセージを表示   
                raise PlotException('Common','C027',("TOF Range",))
            if tof1 > tof2:
                  raise PlotException('Common','C010',("TOF Range End","TOF Range First"))

        except PlotException, ex:
            PlotMessage(self.frame,ex)
            return
        # カーソルを砂時計に変更    
        self.frame.SetCursor(wx.StockCursor(wx.CURSOR_WAIT))    
        # マップデータを再作成    
        self.Z = self.do.RemakeMapData(tof1, tof2, self.hTable)
        # カーソルを戻す   
        self.frame.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))

        self.zMin = int1
        self.zMax = int2
        
        # マップデータからマスクデータ作成
        self.Zm = ma.masked_where(self.Z >= MASKVALUE, self.Z)
        
        # マップを再描画
        self.PlotChart()    
    ##<--[inamura 100621]
    #########################################
    def OnBt2dPlot(self, event):
        """
        2次元プロッタボタン押下イベント処理
        @param   event イベント情報   
        @retval 無し
        """
        try:
            try:
                # PSD No. をテキストボックスから取得
                psd = int(self.psdTx0.GetValue())
            except:
                # エラーメッセージを表示   
                raise PlotException('Common','C026',("PSD No.",))
            # PSD No. の入力範囲チェック
            if psd < 0 or psd > self.maxPSD:
                strPSD = "%d" % self.maxPSD
                raise PlotException('Common','C008',("PSD","0",strPSD))  
            # 有効な検出器か
            if not self.do.IsValid(psd):
                # 無効な検出器
                raise PlotException('FastVis','F005',(self.psdTx0.GetValue(),))
            
        except PlotException, ex:
            PlotMessage(self.frame,ex)
            return
        # PSD のデータを取得
        tof, zz, er = self.do.GetPSDArray(psd)
        
        #Inamura
        if tof==None:
            return
        #--Inamrua
        
        # 2次元マップ用のデータ作成    
        x = arange(0, self.numPixel+1, 1)
        X, Y = meshgrid(x, tof)
        map = (zz, er, X, Y)

        # D2 プロッタが表示されているか
        if self.d2 == None:
            # 2Dプロッタフレームクローズイベントのリスナー登録
            IFEvtProp(100).AddListner('2dclosed',  self.OnNotify2DClose)
            self.d2 = D2Vis.MapD2Frame(self.frame , map, 100, "2D TOF Map") 
        else:
            # データ変更
            IFEvtProp(100).NotifyEvent(self, "changedata", map)        

        # タイトルを取得
        runNo = self.do.GetTitle()
        main = "Run No.: " + runNo
        sub = "PSD No.: %d" % psd
            
        # タイトル設定コマンド送信
        IFEvtProp(100).NotifyEvent(self, "title", (main, sub))
        # スケールラベル設定コマンド送信
        IFEvtProp(100).NotifyEvent(self, "scalelabel", ("Pixel No.", "TOF (micro sec)","Neutrons"))  

    #########################################
    def OnBtSumm(self, event):
        """
        Summボタン押下イベント処理
        @param   event イベント情報   
        @retval 無し
        """ 
        # 積算語にデータをプロッタへ追加
        self.AddToPlot(True)
        
    #########################################    
    def AddToPlot(self, sumFlag):
        """
        Append Sum データをプロッタへ追加
        @param   sumFlag True であれば、積算後に追加   
        @retval 無し
        """ 
        try:
            try:
                # 始点のPSD-ID をテキストボックスから取得
                psd1 = int(self.psdTx1.GetValue())
                px1 = int(self.pxTx1.GetValue())
                self.ppsd = psd1
            except:
                # エラーメッセージを表示   
                raise PlotException('Common','C026',("PSD No. and Pixel No."))
         
            try:
                # 終点のPSD-ID をテキストボックスから取得
                psd2 = int(self.psdTx2.GetValue())
                px2 = int(self.pxTx2.GetValue())
            except:
                # 終点が空白であれば、始点=終点とする
                psd2 = psd1
                px2 = px1
                # 終点テキストボックスをクリアする
                self.psdTx2.SetValue("")
                self.pxTx2.SetValue("")

            # PSD No. の入力範囲チェック
            if psd1 < 0 or psd1 > self.maxPSD or psd2 < 0 or psd2 > self.maxPSD:
                strPSD = "%d" % self.maxPSD
                raise PlotException('Common','C008',("PSD","0",strPSD))    
            # Pixel No. の入力範囲チェック
            if px1 < 0 or px1 >= self.numPixel or px2 < 0 or px2 >= self.numPixel:
                strPx = "%d" % (self.numPixel-1)
                raise PlotException('Common','C008',("Pixel","0",strPx)) 
                    
        except PlotException, ex:
            PlotMessage(self.frame,ex)
            return    

        # 逆転していたなら
        if psd1 > psd2:
            # 入れ替え
            tmp = psd1
            psd1 = psd2
            psd2 = tmp
        # 逆転していたなら
        if px1 > px2:
            # 入れ替え
            tmp = px1
            px1 = px2
            px2 = tmp
        
        # PSD-ID を格納するリストを準備(積算用)
        lstSum = []
        # ヒストグラム を格納するリストを準備
        datList = []
        
        ##[inamura 120115]-->
        Xaxis = self.cbXax.GetSelection() 
        print "Xaxis=",Xaxis
        if Xaxis==-1:
            Xaxis=0
        if Xaxis==0:
            # PSD No. の範囲を繰り返す
            while psd1 <= psd2:
                # 検出器は有効か
                if psd1 in self.hTable and self.do.IsValid(psd1):
                    px = px1
                    # ピクセル範囲を繰り返す
                    while px <= px2:
                        # 積算ならば 
                        if sumFlag:
                            # 積算用PSD-ID リストに追加
                            lstSum.append((psd1, px)) 
                        else:
                            # 個別であれば、ヒストグラムをデータリストに追加
                            dat = self.do.GetHistogram(psd1, px)
                            datList.append(dat) 
                            
                        px +=1
                psd1 +=1
            try:
                # 範囲内に有効なPSDがなければ
                if len(datList) == 0 and len(lstSum) == 0 :
                    raise PlotException('FastVis','F004',()) 
            except PlotException, ex:
                PlotMessage(self.frame,ex)
                return    
        
            # 積算ならば    
            if sumFlag:
                # PSD-ID のリストを渡して積算データを取得
                dat = self.do.IntegrateHistogram(lstSum)
                datList.append(dat)
                
        elif Xaxis==1: ## along PSD
            if sumFlag:
                psdRange=[ psd1,psd2 ]
                pixRange=[ px1, px2 ]
                datList.append( self.do.GetLineFromIntensities(psdRange,pixRange,"PSD") )
            else:
                for p in range(px1,(px2+1)):
                    psdRange = [psd1,psd2 ]
                    pixRange = [ p, p ]
                    datList.append( self.do.GetLineFromIntensities(psdRange,pixRange,"PSD") )

        elif Xaxis==2: ## along Pixel
            if sumFlag:
                psdRange=[ psd1,psd2 ]
                pixRange=[ px1, px2 ]
                datList.append( self.do.GetLineFromIntensities(psdRange,pixRange,"PIXEL") )
            else:
                for p in range(psd1,(psd2+1)):
                    psdRange = [ p, p ]
                    pixRange = [ px1,px2 ]
                    datList.append( self.do.GetLineFromIntensities(psdRange,pixRange,"PIXEL") )
        else:
            pass
        ##<--[inamura 120115]
        # 選択中のプロッタがあれば
        if not self.plot == None:
            # データ追加要求イベント発行
            IFEvtProp(self.plot[1]).NotifyEvent(self, "add", datList)
        else:
            self.psd1 = psd1
            # 選択中のプロッタが無い場合は、新規にプロッタを追加
            self.OnNewPlot(data = datList)
   

    ##############################################
    def OnNewPlot(self, evt=None, data = None):
        """
        新規プロッタの作成
        カレントプロッタの切替
        @param  evt 　イベント
        @param  data ヒストグラムデータ
        @retval 無し
        """     
        # 選択中のプロッタがあれば
        if not self.plot == None:
            # プロッタのアイコン化要求イベント発行
            IFEvtProp(self.plot[1]).NotifyEvent(self, "doiconize", True)

        # プロッタ番号をインクリメント
        self.order  += 1
        
        # プロッタフレームクローズイベントのリスナー登録
        IFEvtProp(self.order).AddListner('plotclose',  self.OnNotifyPlotClose)
        # プロッタフレームアイコン化イベントのリスナー登録
        IFEvtProp(self.order).AddListner('ploticonized',  self.OnNotifyPlotIconized)
        
        # 新しいプロッタウィンドウを開く
        pi = PlotFrame(self.frame, data, self.order)
        self.plot = [pi, self.order]

        # プロッタ一覧に加える文字列を作成
        strPlot = " Plotter(%d)" % self.order
        # プロッタリストに追加
        self.plots.append(self.plot)
        
        # タイトルを取得
        runNo = self.do.GetTitle()
        main = "Run No.: " + runNo
            
        # タイトル設定コマンド送信
        IFEvtProp(self.order).NotifyEvent(self, "title", (main,"*"))
        
        # 表示リストに追加
        self.plist.Append(strPlot)
        # 追加したプロッタを選択状態にする
        pno = len(self.plots)
        self.plist.SetSelection(pno-1) 

    ##############################################    
    def OnListSelect(self, evt):
        """
        リスト選択イベント処理関数
        カレントプロッタの切替
        @param  evt 　イベント
        @retval 無し
        """
        # 選択消失イベントか 2009.05.07 Minakawa Add
        if evt.GetSelection() < 0:
             return
        
        # 選択中のプロッタがあれば
        if not self.plot == None:
            # プロッタのアイコン化要求イベント発行
            IFEvtProp(self.plot[1]).NotifyEvent(self, "doiconize", True) 
               
        # 選択されたプロッタのインスタンスをカレントプロッタとする
        self.plot = self.plots[evt.GetSelection()]
        # プロッタ復元要求イベント発行
        IFEvtProp(self.plot[1]).NotifyEvent(self, "doiconize", False) 

    ##############################################
    def OnLog(self, evt):
        """
        ログメニューイベント処理関数
        ログとリニアのモード切替
        @param  evt 　イベント
        @retval 無し
        """     
        # メニューのチェックを取得し、ログフラグに設定
        self.logFlag = evt.IsChecked()
        ##[inamura 100621]-->
        if self.logFlag:
             if self.zMin <= 0.0:
                  self.zMin = 1.0E-20
        else:
             self.zMin = self.datMin

        self.yminTx.SetValue( ("%g" % self.zMin) )
        ##<--[inamura 100621]
        # マップの再描画
        self.PlotChart()

    ##############################################    
    def OnSlider(self, evt):
        """
        スライダーイベント処理関数
        Z 軸範囲を変更(拡大)
        @param  evt 　イベント
        @retval 無し
        """     
        # スライダーの最大値を取得
        max = self.slider.GetMax()
        # 現在値を取得し、割合に変換
        svalue = max - self.slider.GetValue() + 1
        
        # ログスケールか
        if self.logFlag:
#            self.zMax = self.datMax * log10(svalue) / max
            ##[inamura 100621]-->
            #self.zMax = self.datMax * svalue / max
            if self.datMin <= 0.0:
                 self.zMin = 1.0E-20
            self.zMax = (self.datMax - self.zMin) * svalue / max
            ##<--[inamura 100621]
        # リニアスケール
        else:
            ##[inamura 100621]-->
            #self.zMax = self.datMax * svalue / max
            self.zMax = (self.datMax - self.datMin) * svalue / max
        #if self.zMax < 10.0:
        #    self.zMax = 10.0
            ##<--[inamura 100621]
        # マップの再描画
        ##[inamura 100621]-->
        self.yminTx.SetValue( ("%d" % self.zMin) )
        self.ymaxTx.SetValue( ("%d" % self.zMax) )
        ##<--[inamura 100621]
        self.PlotChart()
        
    #####################################################
    def OnNotifyPlotClose(self, wid, evt, value=None):
        """
        プロッタクローズイベント対応処理
        @param  wid 　 イベント発生元のインスタンス
        @param  evt 　イベントの種類
        @param  value  無し   
        @retval 無し
        """ 
        i = 0
        # インスタンスリストから、クローズしたプロッタを検索
        for pl in self.plots:
            if pl[0] == wid:
               # I/F をクリア 
               IFEvtProp(i+1).InitIF() 
               # インスタンスリストから削除 
               del self.plots[i]
               # 表示リストから削除
               self.plist.Delete(i)
               # クローズしたプロッタがカレントだったなら
               if wid == self.plot[0]:
                   
                   # カレントプロッタを変更
                   pno = len(self.plots)
                   # 他に開いているプロッタがあれば
                   if pno > 0:
                       # リストの最後のプロッタをカレントプロッタとする
                       self.plot = self.plots[pno-1]
                       self.plist.SetSelection(pno-1)
                       # プロッタ復元要求イベント発行
                       IFEvtProp(self.plot[1]).NotifyEvent(self, "doiconize", False) 
                   # 全部のプロッタがクローズされたなら    
                   else:
                       # カレントプロッタ無しとする
                       self.plot = None
                   
               break     
            i +=1

    ###################################################            
    def OnNotifyPlotIconized(self, wid, evt, flag):
        """
        プロッタアイコン化イベント対応処理
        @param  wid 　 イベント発生元のインスタンス
        @param  evt 　イベントの種類
        @param  flag  アイコン化フラグ   
        @retval 無し
        """ 
        i = 0
        # インスタンスリストから、イベントの発生したプロッタを検索
        for pl in self.plots:
            # イベント発生元と同じインスタンスか
            if pl[0] == wid:
                strP = self.plist.GetString(i)
                # アイコン化されたのか
                if flag:
                    # 表示中のリストにアイコン化文字列を付加
                    self.plist.SetString(i, strP+" - Iconized")
                # 復元であれば
                else:
                    # アイコン化文字列を消す
                    self.plist.SetString(i, strP[:-11])
                break
            i +=1          

    ######################################################
    def OnPrint(self,*args):
        """
        Print メニューイベント処理
        print コマンド処理
        印刷ダイアログを表示
        @param  *args イベント情報
        @retval 無し
        """ 
        # プラットフォームを判定     
        if os.name == "posix" or os.name == "mac" :
            # Linux または Mac OS なら
            # 現在の時間よりテンポラリファイル名を作成
            tsec = int(time.time()*10.0)
            tmpf = "map%d.ps" % tsec

            try:
                try:
                    # ポストスクリプトファイルを作成
                    self.canvas.print_figure(tmpf, dpi=600, orientation="landscape")
                except:
                    raise PlotException('Common','C018',(tmpf,))
                # スプーラにファイルを送信
                strCmd = "lpr %s" % tmpf
                ret = os.system(strCmd)
                # エラーがあったなら

                if ret > 0:
                    # エラーメッセージを表示
                    raise PlotException('Common','C023',())
                
            except PlotException, ex:
                PlotMessage(self.frame,ex)
            
        # Linux または Mac OS 以外(Windowsを想定)
        else:
            # 現在の解像度を取得
            dpi = self.canvas.figure.dpi.get()
            # 印刷用に解像度を高くする
            self.canvas.figure.dpi.set(160)
            #用紙をA4に設定
            self.canvas.printerData.SetPaperId(wx.PAPER_A4)
            # 用紙を横方向に設定
            self.canvas.printerData.SetOrientation(wx.LANDSCAPE)
            # マージンを設定
            self.canvas.printer_margin = 1.0
            # 印刷ダイアログを表示

            self.canvas.Printer_Print()
            # 解像度を元に戻す
            self.canvas.figure.dpi.set(dpi)
            # 再描画
            self.canvas.draw()
            
    ######################################################
    def OnSave(self, wid, evt, fname):
        """
        画像保存処理
        @param  wid イベント発生元のインスタンス
        @param  evt イベントタイプ
        @param  fname ファイル名
        @retval 無し
        """ 
        try:
            try:
                self.canvas.print_figure(fname, dpi=300)
            except:
                raise PlotException('Common','C018',(fname,))
        except PlotException, ex:
                PlotMessage(self.parent.frame,ex)
    
    #########################################
    def OnNotify2DClose(self, *args):
        """
        2Dプロッタクローズイベント受信処理
        @param *args　　イベント
        @retval 無し
        """
        # I/F をクリア 
        IFEvtProp(100).InitIF()
        self.d2 = None
                        
    ##############################################        
    def OnClose(self, evt=None):
        """
        デテクタマップ画面クローズ
        @param  evt 　イベント
        @retval 無し
        """
        # 印刷に使用したテンポラリファイルのリストを取得
        tmpfiles = glob.glob("map*.ps")
        # テンポラリファイルを削除
        for tmpf in tmpfiles:
            try:
                # ファイルを削除
                os.remove(tmpf)
            except:
                # 削除 できなかったら(スプーラが使用中?)、次回に削除
                pass
        # メッセージの解放 
        MessageFile().CloseDoc()
        
        # ---> 2008/04/27 Minakawa add
        # 全 1D Plotter のI/F をクリア 
        for i in range(len(self.plots)):
            IFEvtProp(i+1).InitIF()
        # 2Dプロッタが開いているなら
        if self.d2 != None:
            # 2Dプロッタのクローズ要求を出す(孫画面のI/F をクリアさせるため)　
            IFEvtProp(100).NotifyEvent(self, "close")
        # 自分のI/F をクリア
        IFEvtProp(0).InitIF()
        # <--- 2008/04/27 Minakawa add
        
        # 検出器マップ画面のクローズ
        self.frame.Destroy() 
        
#######################################
#  DetectMap
#######################################  
class DetectMap(object):
    """
    Application Class 
    """
    #########################################
    def __init__(self, matrix=None):
        """
        アプリケーションスタート
        @param  matrix  エレメントコンテナマトリックス
        @retval 無し
        """ 
        app = wx.App()
        try:
            if matrix == None:
                raise PlotException('FastVis','F002',())

        except PlotException, ex:
            PlotMessage(None,ex)
        else:    
            
            fig = MapFrame(None, matrix)
            app.MainLoop()


###################################       
if __name__ == '__main__':
    pass
   





# -*- coding: utf-8 -*-
"""

"""

import utsusemi.vis.VisualModuleQBase as UVM
import utsusemi.SAS.vis.UBSlicer as UBS
# 予約語の定義
ret = None
vp1 = None
vp2 = None
vp3 = None
vc = None
vc2 = None
vc3 = None
ec = None
earray = None
DAT = None
ECM = None
ECA = None
vcm = None

###################################################
def DetectMap(matrix=DAT):
        """
        Display Detector Map
        @param  matrix [Def:DAT(ElementContainerMatrix)] ElementContainerMatrix Object
        @retval vp3
        """
        return UVM.DetectMap(matrix)

###################################################
def DetectMapChangeData(vp3=vp3,matrix=DAT):
        """
        Display Detector Map
        @param  matrix [Def:DAT(ElementContainerMatrix)] ElementContainerMatrix Object
        @retval vp3
        """
        UVM.DetectMapChangeData(vp3,matrix)

###################################################
def MPlot(ec=ec):
        """
        Create 1D Plotter and Draw EC
        @param  ec [Def:EC(ElementContainer,ListOfEC)]  ElementContainer
        @param  psd      PSD No.
        @param  pixel    Pixel No.
        @retval vp1    Instance of Plotter
        """
        return UVM.MPlot(ec)

###################################################
def AddToMPlot(vp1=vp1, ec=ec):
        """
        Add EC to 1D Plotter
        @param  vp1 [Def:(vis_retval)] Instance of 1D Plotter
        @param  ec [Def:EC(ElementContainer,ListOfEC)]  ElementContainer
        @retval None
        """
        UVM.AddToMPlot(vp1,ec)

###################################################
def EmptyMPlot():
        """
        Create 1D Plotter and Draw EC without data
        @param  None
        @retval vp1 [Def:vp1] Instance of Plotter
        """
        return UVM.MPlot(None)

###################################################
def M2PlotPlus(matrix=ECA):
        """
        Create 2D Plotter and display 2D Map
        @param  matrix [Def:ECA(ElementContainerArray,ElementContainerMatrix)]  ElementContainerArray, -Matrix
        @retval vp2 [Def:vp2]  Instance of 2D Plotter
        """
        return UVM.M2PlotPlus(matrix)

###################################################
def ChangeDataM2Plot(vp2=vp2, matrix=ECA):
        """
        Change 2D Map of M2Plot
	@param  vp2 [Def:(vis_retval)]     Instance of 2D Plotter
        @param  matrix [Def:ECA(ElementContainerArray,ElementContainerMatrix)]  ElementContainerArray
        @retval None
        """
        UVM.ChangeDataM2Plot(vp2, matrix)

###################################################
def UBSlicer(dat=DAT):
        """
        UBSlicer
        @param  dat [Def:DAT(ElementContainerMatrix)] ElementContainerMatrix
        @retval ubs
        """
        return UBS.UBSlicerFrame(dat,None)

# 関数名登録ディクショナリ
_functions = {"DetectMap":DetectMap,
              "MPlot":MPlot,
              "AddToMPlot":AddToMPlot,
              "EmptyMPlot":EmptyMPlot,
              "M2PlotPlus":M2PlotPlus,
	      "ChangeDataM2Plot":ChangeDataM2Plot,
              "UBSlicer":UBSlicer
	      }

_functionsOrder = [
	"DetectMap",
	"MPlot",
	"AddToMPlot",
        "EmptyMPlot",
	"M2PlotPlus",
	"ChangeDataM2Plot",
	"UBSlicer"
	]




#!/usr/bin/python
# -*- coding: utf-8 -*-

import multiprocessing
from optparse import OptionParser
import os
import re
import platform
import sys

from PySide2 import QtCore, QtGui, QtWidgets
#from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QApplication,QFileDialog,QListWidgetItem,QMessageBox
from PySide2.QtCore import QFile

from ui_PySide_DlgSetUtsusemiEnv import Ui_MainWindow

if sys.platform == 'win32':
    HOME = os.environ.get("USERPROFILE")
elif sys.platform == 'darwin':
    HOME = os.environ.get("HOME")
elif sys.platform == 'linux2':
    HOME = os.environ.get("HOME")
else:
    HOME = os.curdir

# UTSUSEMI_DATA_DIRとUTSUSEMI_USR_DIRのベースディレクトリの初期値
DATA_DIR = os.path.join(HOME, "Documents", "MLF")

# "utsusemienv.cfg"のパス
FILE_CONFIG = os.environ.get("CONFFILE_ENV")
FILE_CONFIG_DIR = os.path.dirname(FILE_CONFIG)

# カレントディレクトリ
CURRENT_DIR = os.getcwd()

# 設定対象の環境変数
UTSUSEMI_INST_CODE = "UTSUSEMI_INST_CODE"
UTSUSEMI_LOG_QUIET = "UTSUSEMI_LOG_QUIET"
UTSUSEMI_DEBUG_MODE = "UTSUSEMI_DEBUG_MODE"
UTSUSEMI_MULTH = "UTSUSEMI_MULTH"
UTSUSEMI_DATA_DIR = "UTSUSEMI_DATA_DIR"
UTSUSEMI_USR_DIR = "UTSUSEMI_USR_DIR"

# ビームライン選択用の辞書
BEAMLINES = {
    "SIK": {'line':  1, 'name': "4SEASONS"},
    "HPN": {'line': 11, 'name': "PLANET"},
    "AMR": {'line': 14, 'name': "AMATERAS"},
    "SAS": {'line': 15, 'name': "TAIKAN"},
}

# Y/N←→真偽値 変換用の辞書
LOG_QUIET = {"Y": True, "N": False}

class Param:
    def __init__(self):
        self.data_dir = os.path.join(DATA_DIR, "DATA")
        self.usr_dir = os.path.join(DATA_DIR, "HOME")
        self.multh = 2 if multiprocessing.cpu_count() >= 2 else 1
        self.log_quiet = True
        self.debug_mode = False
        self.inst_code = "SIK"
        self.loaded = {"UTSUSEMI_" + x:False for x in "MULTH,INST_CODE,LOG_QUIET,DEBUG_MODE,USR_DIR,DATA_DIR".split(",")}

    def load(self, path):
        if not os.path.isfile(path):
            return
        try:
            with open(path) as f:
                d = {}
                for x in f.readlines():
                    if "=" in x:
                        key,val = x.strip().split("=",1)
                        d[key] = re.sub(r'[\"\'\n\r]+', '', val)
                for key,val in d.items():
                    msg = ""
                    if key == "UTSUSEMI_MULTH":
                        try:
                            self.multh = int(val)
                        except:
                            msg = "MULTH: not an integer: >>>" + val + "<<<"
                    elif key == "UTSUSEMI_INST_CODE":
                        self.inst_code = val
                    elif key == "UTSUSEMI_LOG_QUIET":
                        if val in ("Y","N"):
                            self.log_quiet = ("Y" == val)
                        else:
                            msg = "LOG_QUIET: illegal value: >>>" + val + "<<<"
                    elif key == "UTSUSEMI_DEBUG_MODE":
                        if val in ("Y","N"):
                            self.debug_mode = ("Y" == val)
                        else:
                            msg = "DEBUG_MODE: illegal value: >>>" + val + "<<<"
                    elif key == "UTSUSEMI_USR_DIR":
                        self.usr_dir = val
                    elif key == "UTSUSEMI_DATA_DIR":
                        self.data_dir = val.strip('"')
                    if key in self.loaded.keys() and not msg:
                        self.loaded[key] = True
                    if msg:
                        QMessageBox.warning(None,"PARAMETER ERROR",msg)
        except Exception as e:
            QMessageBox.warning(None,"LOAD ERROR","unknown load error: " + str(e))


    def bad_paths(self):
        d = self.data_dir
        u = self.usr_dir
        ret = []
        if not os.path.exists(d) or not os.path.isdir(d):
            ret.append(d)
        if not os.path.exists(u) or not os.path.isdir(u):
            ret.append(u)
        return ret


    def dump(self, path):
        bad = self.bad_paths()
        if bad:
            return bad
        with open(path,"w") as f:
            f.write("UTSUSEMI_INST_CODE="+self.inst_code+"\n")
            f.write("UTSUSEMI_LOG_QUIET="+("Y" if self.log_quiet else "N") +"\n")
            f.write("UTSUSEMI_DEBUG_MODE="+("Y" if self.debug_mode else "N") +"\n")
            f.write("UTSUSEMI_MULTH="+str(self.multh)+"\n")
            f.write("UTSUSEMI_DATA_DIR=\""+self.data_dir+"\"\n")
            f.write("UTSUSEMI_USR_DIR=\""+self.usr_dir+"\"\n")
        return []

class Main(QtWidgets.QMainWindow, Ui_MainWindow):
    def __init__(self, parent):
        #file = QFile(path)
        #file.open(QFile.ReadOnly)
        #loader = QUiLoader()
        #self = loader.load(file)
        super(Main, self).__init__(parent)
        self.param = Param()
        self.setupUi(self)
        self.setup()
        self.connections()
        self.show()


    def setup(self):
        self.items = {}
        for x,vs in BEAMLINES.items():
            s = "BL" + str(vs['line']).zfill(2) + ":" + x + "(" + vs['name'] + ")"
            item = QListWidgetItem(s)
            self.beamList.addItem(item)
            self.items[x] = item
        self.beamList.setCurrentItem(self.items["SIK"])

        self.spinThreads.setRange(1,multiprocessing.cpu_count())
        if FILE_CONFIG is None:
            msg = """You need to define an environment variable 'CONFFILE_ENV'
whose value is the filepath of utsusemienv.cfg"""
            QMessageBox.critical(self,"Missing CONFFILE_ENV",msg)
            sys.exit(1)
        self.param.load(FILE_CONFIG)
        self.textData.setPlainText(self.param.data_dir)
        self.textPriv.setPlainText(self.param.usr_dir)
        self.spinThreads.setValue(self.param.multh)
        if self.param.log_quiet:
            print("QUIET")
            self.checkYes.setChecked(True)
        if self.param.debug_mode:
            print("DEBUG_MODE")
            self.checkDebug.setChecked(True)            
        self.beamList.setItemSelected(self.items[self.param.inst_code],True)

    def connections(self):
        self.buttonBox.accepted.connect(self.save)
        self.buttonBox.rejected.connect(self.quit)
        self.browseData.clicked.connect(self.fdData)
        self.browsePriv.clicked.connect(self.fdPriv)

    def quit(self, force=True):
        if force:
            app.exit()

    def fdData(self):
        UTSUSEMI_DATA_DIR = QFileDialog.getExistingDirectory(self,"DD",self.param.data_dir)
        self.textData.setPlainText(UTSUSEMI_DATA_DIR)

    def fdPriv(self):
        UTSUSEMI_USR_DIR = QFileDialog.getExistingDirectory(self,"DD",self.param.usr_dir)
        self.textPriv.setPlainText(UTSUSEMI_USR_DIR)

    def save(self):
        d1 = self.textData.toPlainText()
        d2 = self.textPriv.toPlainText()
        for d in [d1,d2]:
            if not os.path.exists(d):
                msg = "Not found given folder ({}) . Do you want to make it ?".format(d)
                ret = QMessageBox().warning(None,u"Warning", msg, QMessageBox.Ok,QMessageBox.Cancel )
                if ret == QMessageBox.Ok:
                    try:
                        os.makedirs( d )
                    except:
                        QMessageBox.warning(None,"ERROR", "No permitted.")
                        return
                else:
                    self.quit()
        self.param.data_dir = d1
        self.param.usr_dir = d2
        if not os.path.exists(os.path.join(d2,"ana","xml")):
            try:
                os.makedirs(os.path.join(d2,"ana","xml"))
            except:
                QMessageBox.warning(None,"WARNING", "Failed to make ana/xml ",QMessageBox.Ok)
            try:
                os.makedirs(os.path.join(d2,"ana","tmp"))
            except:
                QMessageBox.warning(None,"WARNING", "Failed to make ana/tmp ",QMessageBox.Ok)
        #self.param.data_dir = self.textData.toPlainText()
        #self.param.usr_dir = self.textPriv.toPlainText()
        self.param.log_quiet = self.checkYes.isChecked()
        self.param.debug_mode = self.checkDebug.isChecked()        
        self.param.multh = self.spinThreads.value()
        citem = self.beamList.currentItem()
        for k,item in self.items.items():
            if item.text == citem.text:
                self.param.inst_code = k
        if not os.path.exists(FILE_CONFIG_DIR):
            os.makedirs(FILE_CONFIG_DIR)
        bad = self.param.dump(FILE_CONFIG)
        if not bad:
            msg = "new settings were saved"
            QMessageBox.information(self,"Confirmation",msg)
            self.quit()
        msg = "illegal path(s): " + ",".join(bad)
        QMessageBox.warning(self,"ILLEGAL PATHS",msg)

if __name__ == '__main__':
    parser = OptionParser(usage=u'%prog [OPTIONS]')
    parser.add_option(
        '-a', '--alert',
        help='Display alert.',
        action='store_true',
        default=False,
        dest='isAlert'
    )
    parser.add_option(
        '-m', '--message',
        help='Alert message to be shown',
        default=None,
        dest='message')
    (option, args) = parser.parse_args()

    app = QApplication(sys.argv)

    if option.isAlert:
        msg = "\n".join([option.message,
                         "",
                         "Run the [Utsusemi Environment Settings] and set up the environment variables."])
        QMessageBox.critical(None,"Missing CONFFILE_ENV",msg)
        sys.exit(1)

    if FILE_CONFIG is None:
        msg = "\n".join(["You need to define an environment variable 'CONFFILE_ENV'"
                         "whose value is the filepath of utsusemienv.cfg"])
        QMessageBox.critical(None,"Missing CONFFILE_ENV",msg)
        sys.exit(1)

    win = Main(None)
    sys.exit(app.exec_())











#! /usr/bin/env python
import sys
import os
import subprocess
import datetime
################################################################################
# Compile script for Manyo-lib release version
#    ver 2023/7/11
#    written by Y. Inamura
#########################################################################
man_script = "Usage :     {:s} <command> [options]\n".format(sys.argv[0])
man_script += """
       command: [depend/man/make/install/clean]
       options: --debug --help --prefix --py-prefix
                --other-modules --numth --pyver
                --rev --without-depend --with-para --without-para
  -------------------------------------------------------
  depend    : depend script
  man       : script to make manual
  make      : make
  install   : make install
  uninstall : make uninstall
  clean     : make clean

  --debug          : debug mode
  --help           : show this help

  --prefix=/hoge/hogege/hoge
                   : set MLFSOFT_PATH,
                     the root path including manyo-lib and python-utsusemi folders
  --py-prefix=/hoge/hogege/hoge
                   : set PYUTSUPREFIX, default ${MLFSOFT_PATH}/python-utsusemi
  --dist-name=Ubuntu
                   : Linux distribution name [RH6,RH7,Ubuntu]
  --other-modules=SAS,DNA
                   : required other modules if need(default Utsusemi)

  --numth=n        : set the number of multi-threading
  --pyver=n.m      : set python version
                     If not set pyver, the version of python executing this is used
  --rev=nnnn       : set revision number if need
  --without-depend : does not run depend script
  --with-para      : use compile option of -j (default is no-use of -j)
  --without-para   : not use compile option of -j

"""

#########################################################################
#   CXX            : compiler
#   PYVER          : version of python
#   NUM_OF_THREADS : number of threading
#   MANYO_PATH     : path to  manyo-core installed
#   INSTALL_PATH   : path to install
#   OTHER_MANYO_PAC: Other Manyo Package names required by compile
#   ENV_FILE_NAME  : file name of shell script to set Manyo environment
#   EXT_LIB_ROOT   : root path to external library required by Manyo-lib
#########################################################################
CXX = "g++"
PYVER = ""
try:
    PYVER = "{:d}.{:d}".format(sys.version_info.major, sys.version_info.minor)
except:
    pass
NUM_OF_THREADS = 8
MLFSOFT_PATH = "/opt/mlfsoft"
MANYO_PATH = MLFSOFT_PATH + "/manyo"
PYUTSU_PREFIX = ""
INSTALL_PATH = MANYO_PATH
OTHER_MANYO_PAC = ["Utsusemi"]
ENV_FILE_NAME = "bashrc.template"
EXT_LIB_ROOT = "/usr/local"
DIST_NAME = ""
if os.path.exists("/etc/redhat-release"):
    f = open("/etc/redhat-release", "r")
    r = f.readline()
    f.close()
    r_list = r.split(" ")
    for i, rel in enumerate(r_list):
        if rel == "release":
            p = r_list[i + 1]
            break
    if p.find("7") == 0 or p.find("8") == 0:
        DIST_NAME = "RH7"
    else:
        DIST_NAME = "RH6"
elif os.path.exists("/etc/os-release"):
    DIST_NAME = "Ubuntu"
else:
    pass

isDebug = False
isWithDepend = True
isCompPara = False
isNotManyoCore = True
utsusemi_rev = 0
isPlacedUgao = False

if "UTSUSEMI_REV" in os.environ:
    utsusemi_rev = int(os.environ["UTSUSEMI_REV"])
else:
    try:
        cp = subprocess.run("git log -1 --date=short|grep 'Date:'", shell=True, capture_output=True, encoding="utf-8")
        if cp.stderr == '':
            latest_date_git = cp.stdout.split()[1]
            latest_date_v = latest_date_git.split("-")
            utsusemi_rev = latest_date_v[0][-2:] + latest_date_v[1] + latest_date_v[2]
    except:
        pass

#############################################
# Check arguments
#############################################

do_command = ""

# print(sys.argv[2:])
unidentified_args = []
if (len(sys.argv) > 1):
    used_args = sys.argv[1:]
    num_of_args = len(used_args)
    ind = 0
    while(True):
        if ind == num_of_args:
            break
        an_arg = used_args[ind]
        ind += 1

        if an_arg in ["depend", "make", "install", "uninstall", "clean", "man"]:
            do_command = an_arg
        elif an_arg == "--debug":
            isDebug = True
        elif an_arg == "--without-depend":
            isWithDepend = False
        elif an_arg == "--without-para":
            isCompPara = False
        elif an_arg == "--with-para":
            isCompPara = True
        elif an_arg.find("--rev=") == 0:
            utsusemi_rev = int(an_arg[6:])
            print("utsusemi_rev = {:s}".format(utsusemi_rev))
        elif an_arg.find("--pyver=") == 0:
            PYVER = an_arg[8:].strip()
            print("PYVER= {}".format(PYVER))
        elif an_arg.find("--numth=") == 0:
            NUM_OF_THREADS = int(an_arg[8:])
            print("NUM_OF_THREADS= {:d}".format(NUM_OF_THREADS))
        elif an_arg.find("--prefix=") == 0:
            MLFSOFT_PATH = an_arg[9:]
            MANYO_PATH = MLFSOFT_PATH + "/manyo"
            INSTALL_PATH = MANYO_PATH
            print("MANYO_PATH= {}".format(MANYO_PATH))
            print("INSTALL_PATH= {}".format(INSTALL_PATH))
        elif an_arg.find("--py-prefix=") == 0:
            PYUTSU_PREFIX = an_arg[12:]
            print("PYUTSU_PREFIX = {}".format(PYUTSU_PREFIX))
        elif an_arg.find("--other-modules=") == 0:
            mod_arg = an_arg[16:]
            OTHER_MANYO_PAC = mod_arg.split(",")
            for a_mod in OTHER_MANYO_PAC:
                print("OTHER_MANYO_PAC= {}".format(a_mod))
        elif an_arg.find("--help") == 0:
            do_command = ""
            break
        elif an_arg.find("--dist-name=") == 0:
            DIST_NAME = an_arg[12:]
            if not (DIST_NAME in ["RH6", "RH7", "Ubuntu", "Ubuntu16", "Ubuntu18", "Ubuntu20"]):
                print(" ERROR : --dist-name is invalid")
                sys.exit(-1)
        else:
            print("Unindentify argument = {}".format(an_arg))
            unidentified_args.append(an_arg)

if (do_command == "") or len(unidentified_args) != 0:
    print(man_script)
    sys.exit(0)

if (do_command == "clean") or (do_command == "uninstall"):
    isWithDepend = False

if (do_command == "make"):
    do_command = ""

if os.path.exists(os.path.join(os.getcwd(), "core")):
    isNotManyoCore = False

#############################################
# Check compile target ( manyo-core or not )
#############################################
if isNotManyoCore:
    PACKAGE_NAME = ""
    makefilepath = os.path.join(os.getcwd(), "manyo", "Makefile")
    if os.path.exists(makefilepath):
        fo = open(makefilepath, "r")
        ll = fo.readlines()
        fo.close()
        for aline in ll:
            print(aline)
            if aline.find("PACKAGE_NAME") != -1:
                PACKAGE_NAME = aline.split("=")[1].strip()
                break
    if PACKAGE_NAME == "":
        print("Makefile is invalid. (No PACKAGE_NAME)")
        sys.exit(1)

if DIST_NAME == "Ubuntu":
    try:
        fo = open("/etc/issue", "r")
        t = fo.readline()
        fo.close()
        v = t.split(' ')
        if v[1].find("22.04") == 0 or v[1].find("20.04") == 0 or v[1].find("19.") == 0:
            DIST_NAME = "Ubuntu20"
        elif v[1].find("18.04") == 0 or v[1].find("17.") == 0 or v[1].find("16.04") == 0 or v[1].find("15.") == 0:
            DIST_NAME = "Ubuntu16"
    except:
        print(" ERROR : failed to distinguish version of Ubuntu")
        sys.exit(-1)
elif DIST_NAME == "Ubuntu18":
    DIST_NAME = "Ubuntu16"


#########################################################################
# execute make or make install
#############################################
OTHER_MAN_PAC = ["Manyo", "MLF"]
for pac in OTHER_MANYO_PAC:
    OTHER_MAN_PAC.append(pac)

EXT_LIB_PATH = os.path.join(EXT_LIB_ROOT, "lib")
EXT_INC_PATH = os.path.join(EXT_LIB_ROOT, "include")

com = "export MANYO_PATH={:s};".format(MANYO_PATH)
if isNotManyoCore:
    com += "export INSTALL_PATH={:s};".format(INSTALL_PATH)
    com += "export LIBRARY_PATH=${MANYO_PATH}/lib:${INSTALL_PATH}:"
    com += "{:s};".format(EXT_LIB_PATH)
else:
    com += "export LIBRARY_PATH=${MANYO_PATH}/lib:"
    com += "{:s};".format(EXT_LIB_PATH)
com += "export LD_LIBRARY_PATH=${LIBRARY_PATH};"
if PYVER != "":
    if DIST_NAME.find("Ubuntu20") == 0:
        com += "export C_INCLUDE_PATH=/usr/include/nexus:/usr/include/hdf5/serial:${MANYO_PATH}/include:"
        com += "/usr/include/python{:s}:{:s}".format(PYVER, EXT_INC_PATH)  # [inamura 201103 for Ubuntu 20.04]
    elif DIST_NAME.find("Ubuntu16") == 0:  # for Ubuntu 16 or 18
        com += "export C_INCLUDE_PATH=/usr/include/hdf5/serial:${MANYO_PATH}/include:"
        com += "/usr/include/python{:s}:{:s}".format(PYVER, EXT_INC_PATH)  # [inamura 201103 for Ubuntu 16.04 and Ubuntu 18.04]
    else:
        com += "export C_INCLUDE_PATH=${MANYO_PATH}/include:"
        com += "{:s}".format(EXT_INC_PATH)
else:
    com += "export C_INCLUDE_PATH=/usr/include/hdf5/serial:${MANYO_PATH}/include:"
    com += "{:s}".format(EXT_INC_PATH)
for m_pac_name in OTHER_MAN_PAC:
    com += ":${MANYO_PATH}/include/"
    com += "{:s}".format(m_pac_name)
com += "; "
com += 'export INC_EXT="'
for m_pac_name in OTHER_MAN_PAC:
    com += "{:s} ".format(m_pac_name)
com += '"; '
com += "export CPLUS_INCLUDE_PATH=${C_INCLUDE_PATH};"
com += "export PYTHONPATH=${MANYO_PATH};"
com += "export PATH=${MANYO_PATH}/bin:${PATH};"
if isNotManyoCore and isWithDepend:
    com += "cd manyo;./depend;cd ..;"

if (do_command != "depend"):
    com += 'make {:s} '.format(do_command)
    if isCompPara:
        com += '-j '
    if DIST_NAME == "":
        DIST_NAME = "Ubuntu"
    com += ' DIST_NAME={:s}'.format(DIST_NAME)
    if PYVER != "":
        com += ' CXX={:s} PyVer={:s} NUM_MULTH={:d}'.format(CXX, PYVER, NUM_OF_THREADS)
    else:
        com += ' CXX={:s} NUM_MULTH={:d}'.format(CXX, NUM_OF_THREADS)
    com += ' UTSUSEMI_REV={}'.format(utsusemi_rev)
    if PYUTSU_PREFIX != "":
        com += ' PYUTSUPREFIX={}'.format(PYUTSU_PREFIX)
    if isPlacedUgao:
        pass
    else:
        com += ' WITOUTUGAO=T'

print("command = {}".format(com))
if isDebug:
    pass
else:
    retcode = os.system(com)
    if retcode:
        sys.stderr.write("Return code '{}', is not 0.\n".format(retcode))
        raise RuntimeError("Error has been caused in building.")


#########################################################################
# make environ file
#############################################
env_temp = """# Utsusemi software environment 
# ({DATE1})

# Utsusemi Environment
export UTSUSEMI_INST_CODE=SIK
export UTSUSEMI_DATA_DIR=/data
export UTSUSEMI_USR_DIR=${{UTSUSEMI_USR_DIR-${{HOME}}}}
export UTSUSEMI_LOG_QUIET=n
export UTSUSEMI_MULTH=2
export UTSUSEMI_DEBUGMODE=n
#export UTSUSEMI_DEFAULT_FONTSIZE=14

# Path to MLF Software
MLFSOFT_PATH={MLFSOFT_PATH}

# Path to Manyo-lib
export MANYO_PATH=${{MLFSOFT_PATH}}/manyo

# PythonPath to utsusemi
PYUTSUSEMI_PATH={PYUTSU_PREFIX}

# External Library path
UTSUSEMI_EXT_LIBRARY_PATH=/usr/local/lib

# Utsusemi Environment
export UTSUSEMI_BASE_DIR=${{PYUTSUSEMI_PATH}}/utsusemi

# Python Path
PYTHONPATH=${{MANYO_PATH}}:${{PYUTSUSEMI_PATH}}:${{UTSUSEMI_BASE_DIR}}/facade
PYTHONPATH=${{UTSUSEMI_BASE_DIR}}/${{UTSUSEMI_INST_CODE}}/facade:${{UTSUSEMI_USR_DIR}}/facade:${{PYTHONPATH}}
export PYTHONPATH

# Shared Library Path
export LD_LIBRARY_PATH=${{MANYO_PATH}}/lib:${{UTSUSEMI_EXT_LIBRARY_PATH}}:${{LD_LIBRARY_PATH}}

# Executable path
export PATH=${{MANYO_PATH}}/bin:${{PATH}}

echo '==========================================='
echo ' Welcome to Utsusemi Environment for '${{UTSUSEMI_INST_CODE}}
echo '                             ver 4.0.{UTSUSEMI_REV}'
echo '==========================================='
"""


if PYUTSU_PREFIX == "":
    PYUTSU_PREFIX = MLFSOFT_PATH + "/python-utsusemi"

dt = datetime.datetime.today()
env_contents = env_temp.format(DATE1="{}/{}/{}".format(dt.year, dt.month, dt.day),
                               MLFSOFT_PATH=MLFSOFT_PATH, PYUTSU_PREFIX=PYUTSU_PREFIX,
                               UTSUSEMI_REV=utsusemi_rev)

if isDebug:
    print(env_contents)
else:
    fo = open(os.path.join(os.getcwd(), ENV_FILE_NAME), 'w')
    fo.write(env_contents)
    fo.close()



#!/usr/bin/env python
from __future__ import print_function
import time
import glob
import numpy
import os
import array
import struct
import getVNRConst #[inamura 180326]

_tagStrLength=16
detectorIDS={"MWPC" :{"daqID":"01","detID":"001"},
             "WLSF1":{"daqID":"01","detID":"001"},
             "WLSF2":{"daqID":"01","detID":"001"},
             "WLSF3":{"daqID":"01","detID":"001"},
             "WLSF4":{"daqID":"01","detID":"001"},
             "RPMT" :{"daqID":"01","detID":"001"},
             "TrigNET":{"daqID":"00","detID":"040"},
             "He":{"daqID":"00","detID":"000"},
             "N2":{"daqID":"00","detID":"002"},
             "0D"   :{"daqID":"00","detID":"254"},
             }
##detectorIDS={"MWPC" :{"daqID":"01","detID":"001"},
##             "WLSF1":{"daqID":"01","detID":"001"},
##             "WLSF2":{"daqID":"01","detID":"001"},
##             "WLSF3":{"daqID":"01","detID":"001"},
##             "WLSF4":{"daqID":"01","detID":"001"},
##             "RPMT" :{"daqID":"01","detID":"001"},
##             "TrigNET":{"daqID":"00","detID":"040"},
##             "0D"   :{"daqID":"00","detID":"254"},
##             }

DIOS=([[16,16]], # DIOS[0]
# unused
# 00010000 mask
# 00010000 available
# mp,pp,mm,pm
[[21,31],[22,31],[25,31],[26,31]],
#[[26,31],[22,31],[25,31],[21,31]], # DIOS[1]
# SF1/SF2
# 00011111 mask
# 00011010 on/on
# 00010110 on/off
# 00011001 off/on
# 00010101 off/off
# mp,pp,mm,pm
[[49,115],[50,115],[81,115],[82,115]],
#[[50,115],[82,115],[49,115],[81,115]])
#[[34,99],[66,99],[33,99],[65,99]]) # DIOS[2]
# SF1/SF3
# 01110011 mask
# 00110010 on/on pp
# 01010010 on/off pm
# 00110001 off/on mp
# 01010001 off/off mm
# swing add 20160401
[[5,31],[6,31],[9,31],[10,31]],
[[33,115],[34,115],[65,115],[66,115]])

offsetPID = 2
XXX="VNR"
#-------------------------------------------------------------------------------
class timeMeasurement:
    def __init__(self,tag=""):
        self.startTime=0
        self.endTime  =0
        self.tag      =tag.ljust(_tagStrLength)
    def start(self):
        self.startTime=time.time()
        if self.tag!="":print(self.tag+">> start")
    def end(self):
        self.endTime  =time.time()
        if self.tag!="":print(self.tag+">> end")
    def show(self):
        if self.tag=="":
            print(str(self.endTime-self.startTime))
        else:
            print(self.tag+">>  %0.4f sec"%(self.endTime-self.startTime))
#-------------------------------------------------------------------------------
def getFilePath(runNo,detector):
#   tm=timeMeasurement("search data path")
#   tm.start()
    runNo_format='%06d' % runNo
    #[inamura 180826]-->
    #if os.name == 'posix' :
    if os.name == 'posix' or os.name == 'nt':
        basepath = getVNRConst.GetDataDir(True)
        #<--[inamura 180826]
        searchName = os.path.join(basepath,XXX+runNo_format+"_????????")
        result=glob.glob(searchName)
        if len(result)>0:
            dname=result[0]
        else:
            print(searchName+" not found!")
            return ""
    else:
        dname=os.path.join(os.getcwd())


    if detector in ["MWPC","WLSF1","WLSF2","RPMT","0D","He","N2","TrigNET"]:
        daqId=detectorIDS[detector]["daqID"]
        detId=detectorIDS[detector]["detID"]
    else:
        print("Wrong detector name!")
        return ""

    fname=XXX+runNo_format+"_"+daqId+"_"+detId+"_???.edb"
    datapath=os.path.join(dname,fname)

    result=glob.glob(datapath)
    if len(result)>0:
        # dname=result[0]
        datapath = os.path.join(dname,result[0])
    else:
        print(datapath+" not found!")
        return ""

#   print "data file path".ljust(_tagStrLength)+">> "+datapath
#   tm.end()
#   tm.show()
    return datapath

def getFilePathNew(runNo,detector):
    """
    New in 1610 ver. (AS)
    getFilePath returns list of files matching the format.
    used in new getEventDataArray
    """
    tm=timeMeasurement("search data path")
    tm.start()
    runNo_format='%06d' % runNo
    #[inamura 180826]-->
    #if os.name == 'posix':
    if os.name == 'posix' or os.name == 'nt':
        basepath = getVNRConst.GetDataDir(True)
        #<--[inamura 180826]
        searchName = os.path.join(basepath,XXX+runNo_format+"_????????")
        result=glob.glob(searchName)
        if len(result)>0:
            dname=result[0]
        else:
            print(searchName+" not found!")
            return ""
    else:
        dname=os.path.join(os.getcwd())

    if detector in ["MWPC","WLSF1","WLSF2","RPMT","0D","He","N2","TrigNET"]:
        daqId=detectorIDS[detector]["daqID"]
        detId=detectorIDS[detector]["detID"]
    else:
        print("Wrong detector name!")
        return ""

    fname=XXX+runNo_format+"_"+daqId+"_"+detId+"_???.edb"
    datapath=os.path.join(dname,fname)

    result=sorted(glob.glob(datapath))
    if len(result)>0:
        dname=result
    else:
        print(datapath+" not found!")
        return ""

    datapath = []
    for d in result:
        print("data file path".ljust(_tagStrLength)+" >> "+d)
    tm.end()
    tm.show()
    return result

def decode_VNR_MWPC(a): # READOUT
    """
    Modified in 1610 ver. (AS)
    used struct.unpack for faster decoding.
    """
    if len(a) < 8:
	 header = '0x0'
         return header, -999
    decode = struct.unpack('8B', a)
    header = hex(decode[0])

    if header == '0xb7': # T0 data (0xb7 = 0x5b<<1|0x01)
        readout = decode[2]
        t0count = (decode[7]<<23) | (decode[6]<<15) | (decode[5]<<7) | ((decode[4]&0xFE)>>1)
        return header, readout, t0count

    elif header == '0xb5': # Neutron data (0xB5 = 0x5a<<1|0x01)
        if decode[7]^0x3f:
	    header = '0x0'
            return header, -999
        tof = (decode[3]<<16) | (decode[2]<<8) | decode[1]  #{[1,2,3]8:15}
        x  = ((decode[5] & 0x0F)<<4) | ((decode[4] & 0xF0)>>4)   #{[4,5]36:47} expired upper 4bit
        ana = (decode[7] & 0x30)>>4 #{[7]60:61}
        err = (decode[7] & 0xC0)>>6 #{[7]62:63}

        return header, tof, 255-decode[6], x, ana, err

    else:
        return header, -999

#-------------------------------------------------------------------------------
def decode_VNR_RPMT(a): # READOUT
    header = hex(ord(a[0])) #{[0]0:7}

    decode = struct.unpack('8B', a)
    header = hex(decode[0])

    if header == '0xb7': # T0 data (0xb7 = 0x5b<<1|0x01)
        readout = decode[2]
        t0count = (decode[7]<<23) | (decode[6]<<15) | (decode[5]<<7) | ((decode[4]&0xFE)>>1)
        return header, readout, t0count

    elif header == '0xb5': # Neutron data (0xB5 = 0x5a<<1|0x01)
        #print '%  ', (" ".join([hex(ord(aa)) for aa in a]))
        if (decode[7] & 0xf0)^0x30:
	    header = '0x0'
            return header, -999

        tof = (decode[3]<<16) | (decode[2]<<8) | decode[1]  #{[1,2,3]8:15}
	x =  (((decode[5] & 0xff) << 1) | (decode[4] >> 7)) #{[4,5]36:47} expired upper 4bit
        y  = (((decode[7] & 0x0f) << 5) | (decode[6] >> 3))  #{[6,7]48:60} expired upper 4bit
        ana = (decode[7] & 0x30)>>4 #{[7]60:61}
        err = (decode[7] & 0xC0)>>6 #{[7]62:63}
	if ( x > 383):
	    x = 383
	if ( y > 383):
	    y = 383
        return header, tof, x - 128, y - 128, ana, err

    else:
        return header, -999

#-------------------------------------------------------------------------------
def decode_VNR_WLSF1(a): # READOUT
    header = hex(ord(a[0])) #{[0]0:7}

    if header == '0xb7': # T0 data (0xb7 = 0x5b<<1|0x01)
        #print '&  ', (" ".join([hex(ord(aa)) for aa in a]))
        readout = long(0xFF & ord(a[2]) ) #{[2]16:23
        t0count = long(0x7FFFFFFF & (ord(a[7])<<23) | (ord(a[6])<<15) | (ord(a[5])<<7) |((ord(a[4])&0xFE)>>1)   )
        #print '&& ',readout, t0count
        return header, readout, t0count

    elif header == '0xb5': # Neutron data (0xB5 = 0x5a<<1|0x01)
        #print '%  ', (" ".join([hex(ord(aa)) for aa in a]))
        tof = long(0xFFFFFF & (ord(a[3])<<16) | (ord(a[2])<<8) | (ord(a[1])) ) #{[1,2,3]8:15}
        x   = long(0xFF & ((ord(a[5])&0x0F)<<4) | ((ord(a[4])&0xF0)>>4) )  #{[4,5]36:47} expired upper 4bit
        y   = long(0xFF & ((ord(a[6]))) ) #{[6,7]48:60} expired upper 4bit
        ana = long(0xF & ((ord(a[7])&0x30)>>4) )#{[7]60:61}
        err = long(0xF & ((ord(a[7])&0xC0)>>6) )#{[7]62:63}
        #print '%% ',tof, x, y, ana, err

        #Area1:0x00, Area2:0x40, Area3:0x80, Area4:0xC0
        if (x&0xC0)==0x00 and (y&0xC0)==0x00:
            return header, tof, long(x&0x3F), long(y&0x3F), ana, err
        else:
            return header, -999

    else:
        return header, -999
#-------------------------------------------------------------------------------
def decode_VNR_WLSF2(a): # READOUT
    header = hex(ord(a[0])) #{[0]0:7}

    if header == '0xb7': # T0 data (0xb7 = 0x5b<<1|0x01)
        #print '&  ', (" ".join([hex(ord(aa)) for aa in a]))
        readout = long(0xFF & ord(a[2]) ) #{[2]16:23
        t0count = long(0x7FFFFFFF & (ord(a[7])<<23) | (ord(a[6])<<15) | (ord(a[5])<<7) |((ord(a[4])&0xFE)>>1)   )
        #print '&& ',readout, t0count
        return header, readout, t0count

    elif header == '0xb5': # Neutron data (0xB5 = 0x5a<<1|0x01)
        #print '%  ', (" ".join([hex(ord(aa)) for aa in a]))
        tof = long(0xFFFFFF & (ord(a[3])<<16) | (ord(a[2])<<8) | (ord(a[1])) ) #{[1,2,3]8:15}
        x   = long(0xFF & ((ord(a[5])&0x0F)<<4) | ((ord(a[4])&0xF0)>>4) )  #{[4,5]36:47} expired upper 4bit
        y   = long(0xFF & ((ord(a[6]))) ) #{[6,7]48:60} expired upper 4bit
        ana = long(0xF & ((ord(a[7])&0x30)>>4) )#{[7]60:61}
        err = long(0xF & ((ord(a[7])&0xC0)>>6) )#{[7]62:63}
        #print '%% ',tof, x, y, ana, err

        #Area1:0x00, Area2:0x40, Area3:0x80, Area4:0xC0
        if (x&0xC0)==0x40 and (y&0xC0)==0x40:
            return header, tof, long(x&0x3F), long(y&0x3F), ana, err
        else:
            return header, -999

    else:
        return header, -999
#-------------------------------------------------------------------------------
def decode_VNR_WLSF3(a): # READOUT
    header = hex(ord(a[0])) #{[0]0:7}

    if header == '0xb7': # T0 data (0xb7 = 0x5b<<1|0x01)
        #print '&  ', (" ".join([hex(ord(aa)) for aa in a]))
        readout = long(0xFF & ord(a[2]) ) #{[2]16:23
        t0count = long(0x7FFFFFFF & (ord(a[7])<<23) | (ord(a[6])<<15) | (ord(a[5])<<7) |((ord(a[4])&0xFE)>>1)   )
        #print '&& ',readout, t0count
        return header, readout, t0count

    elif header == '0xb5': # Neutron data (0xB5 = 0x5a<<1|0x01)
        #print '%  ', (" ".join([hex(ord(aa)) for aa in a]))
        tof = long(0xFFFFFF & (ord(a[3])<<16) | (ord(a[2])<<8) | (ord(a[1])) ) #{[1,2,3]8:15}
        x   = long(0xFF & ((ord(a[5])&0x0F)<<4) | ((ord(a[4])&0xF0)>>4) )  #{[4,5]36:47} expired upper 4bit
        y   = long(0xFF & ((ord(a[6]))) ) #{[6,7]48:60} expired upper 4bit
        ana = long(0xF & ((ord(a[7])&0x30)>>4) )#{[7]60:61}
        err = long(0xF & ((ord(a[7])&0xC0)>>6) )#{[7]62:63}
        #print '%% ',tof, x, y, ana, err

        #Area1:0x00, Area2:0x40, Area3:0x80, Area4:0xC0
        if (x&0xC0)==0x80 and (y&0xC0)==0x80:
            return header, tof, long(x&0x3F), long(y&0x3F), ana, err
        else:
            return header, -999

    else:
        return header, -999
#-------------------------------------------------------------------------------
def decode_VNR_WLSF4(a): # READOUT
    header = hex(ord(a[0])) #{[0]0:7}

    if header == '0xb7': # T0 data (0xb7 = 0x5b<<1|0x01)
        #print '&  ', (" ".join([hex(ord(aa)) for aa in a]))
        readout = long(0xFF & ord(a[2]) ) #{[2]16:23
        t0count = long(0x7FFFFFFF & (ord(a[7])<<23) | (ord(a[6])<<15) | (ord(a[5])<<7) |((ord(a[4])&0xFE)>>1)   )
        #print '&& ',readout, t0count
        return header, readout, t0count

    elif header == '0xb5': # Neutron data (0xB5 = 0x5a<<1|0x01)
        #print '%  ', (" ".join([hex(ord(aa)) for aa in a]))
        tof = long(0xFFFFFF & (ord(a[3])<<16) | (ord(a[2])<<8) | (ord(a[1])) ) #{[1,2,3]8:15}
        x   = long(0xFF & ((ord(a[5])&0x0F)<<4) | ((ord(a[4])&0xF0)>>4) )  #{[4,5]36:47} expired upper 4bit
        y   = long(0xFF & ((ord(a[6]))) ) #{[6,7]48:60} expired upper 4bit
        ana = long(0xF & ((ord(a[7])&0x30)>>4) )#{[7]60:61}
        err = long(0xF & ((ord(a[7])&0xC0)>>6) )#{[7]62:63}
        #print '%% ',tof, x, y, ana, err

        #Area1:0x00, Area2:0x40, Area3:0x80, Area4:0xC0
        if (x&0xC0)==0xC0 and (y&0xC0)==0xC0:
            return header, tof, long(x&0x3F), long(y&0x3F), ana, err
        else:
            return header, -999

    else:
        return header, -999


#-------------------------------------------------------------------------------
def decode_VNR_TrigNET(a):
    header = hex(ord(a[0])) #{[0]0:7}
    #print header

    if header == '0x54': # Digital data
        #print '1% ', (" ".join([hex(ord(aa)) for aa in a]))
        tof = long(0xFFFFFF & (ord(a[3])<<16) | (ord(a[2])<<8) | (ord(a[1])) )#/40000000. #{[1,2,3]8:15}
        sid = long(0xFF & (ord(a[4])) )
        dio = long(0xFF & (ord(a[7])) )
        #print '2% ', tof, sid, dio
        return header, tof, sid, dio

    elif  header == '0x5b': # T0
        #print '1&  ', (" ".join([hex(ord(aa)) for aa in a]))
        crate = long(0x00 & (ord(a[1])) ) #{[1]8:15
        module = long(0x00 & (ord(a[2])) ) #{[2]16:23
        #t0count = long(0xFFFFFFFFFF & (ord(a[7])<<36) | (ord(a[6])<<24) | (ord(a[5])<<16) | (ord(a[4])<<8) | (ord(a[3])) )
        t0count = long(0xFFFFFFFFFF & (ord(a[3])<<36) | (ord(a[4])<<24) | (ord(a[5])<<16) | (ord(a[6])<<8) | (ord(a[7])) )
        #print '2& ',crate,module, t0count
        return header, crate,module, t0count

    elif  header == '0x5c': # Instrument clock
        #print '1#  ', (" ".join([hex(ord(aa)) for aa in a]))
        sec = long(0x3FFFFFFF & (ord(a[1])<<22) | (ord(a[2])<<14) | (ord(a[3])<<6) | (ord(a[4])&0x3F) ) #{[1]8:15
        subsec = long(0x7FFF & ((ord(a[4])&0xC0)>>6<<13) | (ord(a[5])<<5) | (ord(a[6])&0x1F) ) #{[2]16:23
        submsec = long(0x07FF & ((ord(a[6])&0xF8)>>5<<8) | (ord(a[7])) )
        #print '2# ', sec, subsec, submsec
        return header , sec, subsec, submsec

#-------------------------------------------------------------------------------
def processTrigNet(inputParam, dioSortingList, dio):
    """
    New in 1610 ver. (AS)
    Extracted from getEventDataArray(old)
    """

    if isinstance(inputParam,int):
        filename=getFilePath(inputParam,'TrigNET')
        print(filename)

        firstPID = -1 # local
        beginPID = -1 # local
        endPID = -1 # local

        DIO = -1 # local

        currDIO = -1 # local
        indexDIO = -1 #local

        iDIO = -1 # local
        nDIO = -1 # local
    

        #--------------------------
        f = open(filename, "rb")
        while 1:
            a=f.read(8)
            if len(a)==0:
                break
        #--------------------------
            b=decode_VNR_TrigNET(a)
            #print b

            if b[1]==-999:
                continue

            if b[0]=='0x5b':

                PID = long(b[3])
                if firstPID == -1:
                    firstPID = PID

                if DIO != -1 and (beginPID == -1 or endPID != -1) and nDIO != 0:
                    beginPID = -1
                    endPID = -1
                    nDIO = 0
                    currDIO = -1
                    indexDIO = -1
                elif DIO != -1 and (beginPID == -1 or endPID != -1) and nDIO == 0:
                    beginPID = PID
                    endPID = -1
                    nDIO = 0
                    currDIO = DIO
                    indexDIO = iDIO
                    #print '**********  Relally Begin PID: ', beginPID, endPID, currDIO, indexDIO

                elif beginPID!=-1 and  nDIO != 0:
                    endPID = PID-1
                    if endPID-beginPID-offsetPID>=0:
                        dioSortingList.append((indexDIO, beginPID-firstPID+offsetPID ,beginPID+offsetPID, endPID-beginPID-offsetPID, DIOS[dio][indexDIO][0],DIOS[dio][indexDIO][1] ))
                        #print '*** Begin-End PID:   ', beginPID,'-' ,endPID, indexDIO, currDIO, DIOS[dio][indexDIO]

            if b[0]=='0x54':
                index =0
                for dio1 in DIOS[dio]:
                    if (b[3]&dio1[1])==dio1[0]:
                        DIO=dio1[0]
                        if DIO != currDIO:
                            nDIO += 1
                            iDIO = index
                    index+=1
                    #print '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ', indexDIO, firstPID, beginPID, endPID, PID


        if beginPID != -1:
            if PID-beginPID-offsetPID>=0:
                dioSortingList.append((indexDIO, beginPID-firstPID+offsetPID ,beginPID+offsetPID, PID-beginPID-offsetPID, DIOS[dio][indexDIO][0],DIOS[dio][indexDIO][1] ))
        #--------------------------
        f.close()
        #--------------------------
    return dioSortingList

def createT0Count (dio, dioSortingList, T0Count):
    if dio == 0:
        c0=0
        for aaa in dioSortingList:
            c0+=aaa[3]+1
        T0Count.append(c0)
    elif dio != -1:
        c0=0
        c1=0
        c2=0
        c3=0
        for aaa in dioSortingList:
            if aaa[0]==0: c0+=aaa[3]+1
            if aaa[0]==1: c1+=aaa[3]+1
            if aaa[0]==2: c2+=aaa[3]+1
            if aaa[0]==3: c3+=aaa[3]+1

        T0Count.append(c0)
        T0Count.append(c1)
        T0Count.append(c2)
        T0Count.append(c3)
    return T0Count

def getEventDataArray(inputParam,detector="MWPC",pixels=(256,256),dio=-1, nT0start=0, nT0end=-1):
    """
    Modified in 1610 ver. (AS)
    used array.array to conserve memory
    """
    if nT0end == -1:
        nT0end = 2 ** 32  # nT0 end set to very large number
    tm=timeMeasurement("data read")
    tm.start()

    print("dio in getEventDataArray", dio)

    dioSortingList=[]
    if dio != -1:
        dioSortingList = processTrigNet(inputParam, dioSortingList, dio)

    T0Count = []
    T0Count = createT0Count(dio, dioSortingList, T0Count)

    if isinstance(inputParam,int):
        filelist=getFilePathNew(inputParam,detector)
    elif isinstance(inputParam,str) or isinstance(inputParam,unicode):
        filelist=glob.glob(inputParam)
    
    PX=pixels[0]
    PY=pixels[1]

    if dio == -1 or dio == 0:

        data=[[[array.array('l') for i in xrange(PX)] for j in xrange(PY)]]
    else:
        data=[[[array.array('l') for i in xrange(PX)] for j in xrange(PY)],
              [[array.array('l') for i in xrange(PX)] for j in xrange(PY)],
              [[array.array('l') for i in xrange(PX)] for j in xrange(PY)],
              [[array.array('l') for i in xrange(PX)] for j in xrange(PY)]]

    indexDIO=0
    iDIO = 0 # flag
    nDIO = 0 # count bet. each periods
    fDIO = 0 # flag for increasing an index
    nT0 = 0 # for dio=-1
    nT0Range = 0 # for T0 range

    func_name = "".join(['decode_',XXX,'_',detector])
    func = globals()[func_name]

    nT0_in_range = False # Flag if nT0 is in parameters set by option
    for filename in filelist:
        with open(filename, "rb") as f:
            #i1=0
            #i2=0

            while 1:
                a=f.read(8)
                if len(a) < 8:
                    break
                b = func(a)
                if b[1]==-999:
                    #AS Start 20160125 4 byte back for many bad data reduction.
                    f.seek(-4,1)
                    #AS End
                    continue

                if b[0]=='0xb7':
		    # nT0 range check (1610 AS)
                    nT0Range+=1
                    if nT0Range > nT0end:
                        break
                    elif nT0Range >= nT0start:
                        nT0_in_range = True
		    # nT0 range check end
                    if dio == -1:
                        nT0+=1
                    else:
                        if len(dioSortingList)==0:
                            break

                        if fDIO==1:
                            indexDIO+=1
                            nDIO = 0
                            fDIO = 0
                            if indexDIO >= len(dioSortingList):
                                break

                        PID = long(b[2])
                        #print '################### ', indexDIO, len(dioSortingList)
                        if dioSortingList[indexDIO][1]>PID:
                        #print 'before PID ', PID
                            iDIO = 0
                            nDIO = 0

                        elif dioSortingList[indexDIO][1]+dioSortingList[indexDIO][3]>=PID:
                            iDIO = 1
                            #print '*** ', PID, dioSortingList[indexDIO]

                        if dioSortingList[indexDIO][1]+dioSortingList[indexDIO][3]==PID:
                            fDIO = 1

                elif b[0] == '0xb5' and nT0_in_range: # nT0 flag check added (1610 AS)
                    if b[2]<PX and b[3]<PY:
                        if dio == -1:
                            data[0][b[3]][b[2]].append(b[1])
                        elif dio == 0:
                            if iDIO == 1:
                                nDIO+=1
                                data[0][b[3]][b[2]].append(b[1])
                        else:
                            if iDIO == 1:
                                nDIO+=1
                                data[dioSortingList[indexDIO][0]][b[3]][b[2]].append(b[1])


    if dio == -1:
        T0Count.append(nT0)

    datalist=[]

    if dio == -1 or dio == 0:
        dl = []
        dl=[[numpy.array(data[0][j][i]) for i in xrange(PX)] for j in xrange(PY)]

        datalist.append(dl)
    else:
        for k in [0,1,2,3]:
            dl = []
            dl=[[numpy.array(data[k][j][i]) for i in xrange(PX)] for j in xrange(PY)]

            datalist.append(dl)

    tm.end()
    tm.show()

    print(T0Count)
    return datalist, T0Count
#-------------------------------------------------------------------------------
def getDim2Tofs(inputParam,bininfo=(0.0,40000.0,100.0),positionRegion=None,
                detector="MWPC",pixels=(256,256)):
    if isinstance(inputParam,int):
        data = getEventDataArray(inputParam,detector,pixels)
    elif isinstance(inputParam,str) or isinstance(inputParam,unicode):
        data = getEventDataArray(inputParam,detector,pixels)
    elif isinstance(inputParam,list) or isinstance(inputParam,tuple)\
               or isinstance(inputParam,numpy.ndarray):
        data = inputParam
    else:
        return []

    tm=timeMeasurement("create histogram")
    tm.start()

    bin_ini=bininfo[0]
    bin_end=bininfo[1]
    bin_wid=bininfo[2]
    bin_num=int((bininfo[1]-bininfo[0])/bininfo[2])

    PX=pixels[0]
    PY=pixels[1]
    if positionRegion==None:
        retData=[[[] for i in xrange(PX)] for j in xrange(PY)]
        for i in xrange(PY):
            for j in xrange(PX):
                histDatas=numpy.histogram(data[i][j],
                                          bins=bin_num,
                                          normed=False,
                                          range=(bin_ini,bin_end))
                if i==0 and j==0:
                    histDataX= (histDatas[1][1:]+histDatas[1][:-1])*0.5
                histDataY= histDatas[0]
                retData[i][j]=[histDataX,histDataY]
        tm.end()
        tm.show()
        return retData
    else:
        if len(positionRegion)==2:
            roi=_getPositionList(positionRegion)
        elif len(positionRegion)>2:
            roi=positionRegion

        histDataX=numpy.zeros(bin_num)
        histDataY=numpy.zeros(bin_num)
        for (pos0,pos1) in roi:
            if 0 <= pos0 < PX and 0 <= pos1 < PY:
                histDatas=numpy.histogram(data[pos1][pos0],
                                          bins=bin_num,
                                          normed=False,
                                          range=(bin_ini,bin_end))
                histDataX =(histDatas[1][1:]+histDatas[1][:-1])*0.5
                histDataY+=histDatas[0]
            else:
                print("position parameter is invalid")

        tm.end()
        tm.show()
        return histDataX,histDataY
#-------------------------------------------------------------------------------
def getDim2Integral(inputParam,
                         timeRegion=[-numpy.inf,numpy.inf],
                         positionRegion=None,detector="MWPC"):
    if isinstance(inputParam,int):
        filename=getFilePath(inputParam,detector)
        data = getDim2Tofs(filename)
    elif isinstance(inputParam,str) or isinstance(inputParam,unicode):
        data = getDim2Tofs(inputParam)
    elif isinstance(inputParam,list) or isinstance(inputParam,tuple) \
                    or isinstance(inputParam,numpy.ndarray):
        data = inputParam
    else:
        return []

    tm=timeMeasurement("create 2d-histogram")
    tm.start()

    if positionRegion!=None:
        if len(positionRegion)==2:
            roi=_getPositionList(positionRegion)
        elif len(positionRegion)>2:
            roi=positionRegion

    tdata = data[0][0][0]
    region_min,region_max = _getIndexRegion(tdata,timeRegion[0],timeRegion[1])
    PY=len(data)
    PX=len(data[0])

    retData=[[0 for i in xrange(PX)] for j in xrange(PY)]
    if positionRegion==None:
        for i in xrange(PY):
            for j in xrange(PX):
                idata=data[i][j][1][region_min:region_max]
                retData[i][j]=numpy.sum(idata)
    else:
        for (pos0,pos1) in roi:
            if 0 <= pos0 < PX and 0 <= pos1 < PY:
                idata=data[pos1][pos0][1][region_min:region_max]
                retData[pos1][pos0]=numpy.sum(idata)

    tm.end()
    tm.show()

    return numpy.array(retData)
#-------------------------------------------------------------------------------
def getProjection(inputParam,direction="x",
                           timeRegion=[-numpy.inf,numpy.inf],
                           positionRegion=None,detector="MWPC"):
    if isinstance(inputParam,int):
        data = getDim2Integral(inputParam,timeRegion,positionRegion,detector)
    elif isinstance(inputParam,str) or isinstance(inputParam,unicode):
        data = getDim2Integral(inputParam,timeRegion,positionRegion,detector)
    elif isinstance(inputParam,list) or isinstance(inputParam,tuple)\
            or isinstance(inputParam,numpy.ndarray):
        data = inputParam
    else:
        return []

    PY=len(data)
    PX=len(data[0])

    if direction=="x" or direction=="X":
        histY=[]
        histX=numpy.arange(PX)
        for j in xrange(PX):
            val=0
            for i in xrange(PY):
                val+=data[i][j]
            histY.append(val)
        histY=numpy.array(histY)
    elif direction=="y" or direction=="Y":
        histY=[]
        histX=numpy.arange(PY)
        for i in xrange(PY):
            val=0
            for j in xrange(PX):
                val+=data[i][j]
            histY.append(val)
        histY=numpy.array(histY)

    return histX,histY
#-------------------------------------------------------------------------------
def _getPositionList(position):
    if isinstance(position[0],list) or isinstance(position[0],tuple):
        x=range(position[0][0],position[0][1]+1)
    elif isinstance(position[0],int):
        x=[position[0]]
    else:
        x=0

    if isinstance(position[1],list) or isinstance(position[1],tuple):
        y=range(position[1][0],position[1][1]+1)
    elif isinstance(position[1],int):
        y=[position[1]]
    else:
        y=0

    roi = [(i,j) for i in x for j in y]
    return roi
#-------------------------------------------------------------------------------
def _getIndexRegion(tdata,region1,region2):

    region1,region2=_swap(region1,region2)
    maxSize=len(tdata)
    region_min=maxSize-1
    for i in xrange(maxSize):
        if tdata[i]>region1:
            region_min=i
            break

    region_max=0
    for i in xrange(maxSize):
        if tdata[maxSize-i-1]<region2:
            region_max=maxSize-i-1
            break
    return region_min,region_max
#-------------------------------------------------------------------------------
def _swap(x1,x2):
    if x1>x2:
        tmp=x1
        x1=x2
        x2=tmp
    return x1,x2
#-------------------------------------------------------------------------------
def getEventDataArray0D(inputParam):
    tm=timeMeasurement("data read")
    tm.start()

    if isinstance(inputParam,int):
        filename=getFilePath(inputParam,"0D")
    elif isinstance(inputParam,str) or isinstance(inputParam,unicode):
        filename=inputParam

    f=open(filename,"rb")
    s=f.read()
    f.close()
    data=[]
    print("This data has "+str(len(s)) +" Blocks")

    initial = 0
    final   = len(s)
    for i in xrange(initial,final,8):
        if 0x5a==ord(s[i]): # neutron
            tof=long(ord(s[i+1])<<16)+(long(ord(s[i+2])<<8))+(long(ord(s[i+3])))
            data.append(tof)
    data=numpy.array(data)
    tm.end()
    tm.show()
    return data
#-------------------------------------------------------------------------------
def getDim0Tof(inputParam,bininfo=(0.0,40000.0,100.0)):
    if isinstance(inputParam,int):
        data = getEventDataArray0D(inputParam)
    elif isinstance(inputParam,str) or isinstance(inputParam,unicode):
        data = getEventDataArray0D(inputParam)
    elif isinstance(inputParam,list) or isinstance(inputParam,tuple) \
               or isinstance(inputParam,numpy.ndarray):
        data = inputParam
    else:
        return []

    tm=timeMeasurement("create histogram")
    tm.start()

    bin_ini=bininfo[0]*40.0
    bin_end=bininfo[1]*40.0
    bin_wid=bininfo[2]*40.0
    bin_num=int((bininfo[1]-bininfo[0])/bininfo[2])

    histDatas=numpy.histogram(data,
                              bins=bin_num,
                              normed=False,
                              range=(bin_ini,bin_end))

    histDataX= (histDatas[1][1:]+histDatas[1][:-1])*0.0125
    histDataY= histDatas[0]

    tm.end()
    tm.show()
    return histDataX,histDataY

if __name__ == '__main__':
    import sys

    if 3 == len(sys.argv):
        aa =getEventDataArray(int(sys.argv[1]),dio=int(sys.argv[2]))
#        aa =getEventDataArray(int(sys.argv[1]),dio=int(sys.argv[2]), nT0start=100000, nT0end=150000)
        print(aa[1])

#    aa =getEventDataArray(sys.argv[1], dio=-1, nT0end=10000)
#    print aa[1]




#!/usr/bin/python

#import FuncComb
import Manyo
import BuildInFunc

expr=""
for i in range(5):
    expr = expr + " g"
print expr

#        0    1    2    3    4    5    6    7    8    9   10   11   12   13   14
param=[1.0, 0.0, 1.0, 2.0, 0.0, 2.0, 3.0, 0.0, 3.0, 4.0, 0.0, 4.0, 5.0, 0.0, 5.0]
error=[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
pv=Manyo.DoubleVector()
ev=Manyo.DoubleVector()

#parser=FuncComb.FuncParser(expr);
#funcList=parser.parse()
#funcList.clear()

func=BuildInFunc.FuncComb()
func.dump()
func=BuildInFunc.FuncComb(expr)
func.dump()
func=BuildInFunc.FuncComb(expr, param)
func.dump()
func=BuildInFunc.FuncComb(expr, param, error)
func.dump()

xmin=-5.0
xmax= 5.0
div=100
#delta=(xmax - xmin)/div
delta=(xmax - xmin) / div
print "xmin=", xmin, " xmax=", xmax, " div=", div, " delta=", delta
for i in range(div+1):
    x=xmin+delta*i
    print i, x, func.eval(x), func.der1st(x), func.der2nd(x), func.evalError(x)
    for j in range(5):
        print i, j, x, func.evalComponent(x, j), func.der1stComponent(x, j), func.der2ndComponent(x, j), func.evalErrorComponent(x, j)


#x=[0.0, 0.1, 0.2, 0.3]
x=Manyo.DoubleVector()
x.push_back(0.0)
x.push_back(0.1)
x.push_back(0.3)
x.push_back(0.4)

values=func.eval(x)
print values.size()
for i in range( values.size() ):
    print i, x(i), values(i)



#!/usr/bin/python

import Manyo
import Adv

import PlotElementContainer

########  the linear conbination of built in functions and its parameters ########
# an string expression for the linear combination of built-in functions. ####
# three gaussian functions 
expr="g g g"

# a python list of parameters for the functions,
# paramList[0] : the height for the first gaussian
# paramList[1] : the peak position for the first gaussian
# paramList[2] : the HWHM for the first gaussian
#
# paramList[3] : the height for the second gaussian
# paramList[4] : the peak position for the second gaussian
# paramList[5] : the HWHM for the second gaussian
#
# paramList[6] : the height for the third gaussian
# paramList[7] : the peak position for the third gaussian
# paramList[8] : the HWHM for the third gaussian
paramList =[1.0, 0.0, 1.0, 1.0, 5.0, 1.5, 1.0, 10.0, 1.5]

# a python list of parameter errors for the functions
# paramErrList[i] : error (uncertinity) for  paramList[i]
paramErrList=[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  0.0, 0.0]


#########  bin  ########
lower=-5.0 # the lower bound of a domain
upper=15.0 # the upper bound of a domain
divnum=100 # the number of bins
delta=(upper-lower)/divnum # the width of bins

vtools=Manyo.VectorTool()
bin=vtool.MakeVectorDouble(divnum+1)
for i in range(divnum+1):
    bin[i]=lower + delta*i

########  evaluate the values of functions and export the results to an ElementContainer
funcToEC=Adv.FuncCombToElementContainer(expr, paramList, paramErrList)
keys=["x", "y", "e"]
units=["", ""]   #  units for "x" and "y"
ecc = funcToElementContainer(keys, units, bin)
ecl = funcToElementContainer(keys, units, bin, 0.0)
ecu = funcToElementContainer(keys, units, bin, 1.0)

########  plot results  ########
plotter=PLotElementContainer.PlotElementContainer()
plotter.setSource(ece)
plotter.plot()
plotter.setSource(ecl)
plotter.plot()
plotter.setSource(ecu)
plotter.plot()



#!/usr/bin/python
import wx
import wx.lib.newevent

import xrc.base
import time

import execCommandCommon as ecc
(ExecCommandEvent, EVT_EXEC_COMMOND) = wx.lib.newevent.NewEvent()

#-----------------------------------------------------------------------------------
class MainPanel(xrc.base.PanelBase):
    def __init__(self, parent):
        xrc.base.PanelBase.__init__(self, parent, 'demo01.xrc', 'base')
        btn=self.getControl("Ctrl_Btn")
        btn.Bind(wx.EVT_BUTTON,self.onBtn)
        self.Bind(EVT_EXEC_COMMOND,self.progress)
        self.Fit()
#-----------------------------------------------------------------------------------
    def onBtn(self,event):
        target=self.getControl("Ctrl_Target")
        try:
            val=int(target.GetValue())
        except:
            val=5

        ecc.ExecCommandCommon(self.counter,event,args=(val),objs=[target]) # *******
#-----------------------------------------------------------------------------------
    def progress(self,event):
        present=self.getControl("Ctrl_Present")
        present.SetLabel(event.text)
#-----------------------------------------------------------------------------------
    def counter(self,wait=10):
        for i in range(wait):
            text = str(i+1)+" / "+str(wait)
            evt = ExecCommandEvent(text = text)
            wx.PostEvent(self, evt)
            time.sleep(1)
#-----------------------------------------------------------------------------------
class MainFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, wx.ID_ANY, 'demo01')

        sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)
        self.panel = MainPanel(self)
        sizer.Add(self.panel, flag = wx.ALL | wx.EXPAND, border = 1, proportion = 1)
        self.Fit()
        return
#-----------------------------------------------------------------------------------
if __name__ == "__main__":
    app = wx.App()
    frame = MainFrame()
    app.SetTopWindow(frame)
    frame.Show()
    app.MainLoop()
#-----------------------------------------------------------------------------------



from __future__ import print_function
import time,sys,os,math
import Manyo as mm
import Manyo.Utsusemi as mu
import uGao.MPlot as mp


ver = float(sys.version_info[0])+float(sys.version_info[1])*0.1
if ver<2.5:
    from elementtree.ElementTree import ElementTree
else:
    from xml.etree.ElementTree import ElementTree

def GetECPeakInfo(ec,note,emin,emax,delta_bin,):
    """
    @param ec       (ElementContainer)
    @param note     (string)
    @param emin     (float)
    @param emax     (float)
    @param delta_bin     (int)
    @retval result
    """
    import datetime as dt
    xbin = ec.PutX()
    ybin = ec.PutY()
    ebin = ec.PutE()

    xbin_min = 0
    xbin_max = 0
    sum_val = 0
    sum_val_e = 0
    sum_val_e2 = 0
    net_val = 0
    net_val_e = 0

    back_val1_buf = 0
    back_val1_e2 = 0
    back_val2_buf = 0
    back_val2_e2 = 0

    back_val = 0
    back_val_e = 0

    ymax = 0
    ymax_i=0
    y_back = 0
    y_back_e = 0

    ymax_mean_buf=0
    ymax_mean=0
    ymax_mean_e2=0
    ymax_mean_e=0

    fwhm=0
    fwhm_e=0

    for i in range(len(xbin)):
        if xbin[i] < emin:
                xbin_min = i + 1
        if xbin[i] < emax:
                xbin_max = i + 2
 
    for i in range(xbin_min,xbin_max):
        sum_val += ybin[i]
        sum_val_e2 += ebin[i]*ebin[i]
        if ymax<ybin[i]:
             ymax=ybin[i]
             ymax_i=i
    sum_val_e = math.sqrt(sum_val_e2)
    for i in range(ymax_i - delta_bin , ymax_i + delta_bin):
        ymax_mean_buf += ybin[i]
        ymax_mean_e2 += ebin[i]*ebin[i] 

    ymax_mean = ymax_mean_buf/(delta_bin*2)
    ymax_mean_e = math.sqrt(ymax_mean_e2)/(delta_bin*2)

    for i in range(xbin_min - delta_bin , xbin_min + delta_bin):
        back_val1_buf += ybin[i]
        back_val1_e2 += ebin[i]*ebin[i]
    for i in range(xbin_max - delta_bin , xbin_max + delta_bin):
        back_val2_buf += ybin[i]
        back_val2_e2 += ebin[i]*ebin[i]

    y_back = (back_val1_buf + (back_val2_buf - back_val1_buf)/(xbin_max-xbin_min)*(ymax_i-xbin_min))/(delta_bin*2)
    y_back_e = (math.sqrt(back_val1_e2) + math.sqrt(back_val1_e2+back_val2_e2)/(xbin_max-xbin_min)*(ymax_i-xbin_min))/(delta_bin*2)

    back_val = (back_val1_buf+back_val2_buf)/(delta_bin*4)*(xbin_max-xbin_min)
    back_val_e = math.sqrt(back_val1_e2+back_val2_e2)/(delta_bin*4)*(xbin_max-xbin_min)

    net_val = sum_val - back_val
    net_val_e = math.sqrt(sum_val_e2 + back_val_e * back_val_e)

    peak_intenisity = ymax_mean -y_back
    peak_intenisity_e = math.sqrt(ymax_mean_e*ymax_mean_e + y_back_e*y_back_e)

    fwhm = net_val/peak_intenisity/math.sqrt(2*math.pi)*2*math.sqrt(2*math.log(2))*(xbin[1]-xbin[0])
    fwhm_e = net_val/peak_intenisity *math.sqrt((net_val_e**2)/(net_val**2) + (peak_intenisity_e**2)/(peak_intenisity**2))/math.sqrt(2*math.pi)*2*math.sqrt(2*math.log(2))*(xbin[1]-xbin[0])

    print("")
    print("////////////////////////////////////////////")
    print("///    Integral Counting result          ///")
    print("xbin start     = %.2f"%(emin))
    print("xbin end       = %.2f"%(emax))
    print("peak intensity = %.2e, error = %.2e"%(ymax_mean,ymax_mean_e))
    print("gross count    = %.2e, error = %.2e"%(sum_val,sum_val_e))
    print("back count     = %.2e, error = %.2e"%(back_val,back_val_e))
    print("net count      = %.2e, error = %.2e"%(net_val,net_val_e))
    print("fwhm           = %.2e, error = %.2e"%(fwhm,fwhm_e))
    print("////////////////////////////////////////////")

    result = []

    result.append(ymax_mean)
    result.append(fwhm)
    result.append(y_back)

    day=dt.datetime.now()

    filename = "PeakInfo"+str(day.year)+"_"+str(day.month)+"_"+str(day.day)+".txt"
    #filepath = os.path.join(os.environ["HOME"],"ana","tmp",filename)
    filepath = os.path.join(mu.UtsusemiEnvGetUserDir(),"ana","tmp",filename)
    flug = os.path.exists(filepath)

    if flug == False:
        fw = open(filepath, 'w')
        fw.write('time\tNo\txbin_start\txbin_end\tpeak_intensity\terror\tgross_count\terror\tback_count\terror\tnet_count\terror\tFWHM\terror\tnote\n')
        line_num=1
    else:
        fr = open(filepath, 'r')
        line_num = len(fr.readlines())
        fr.close()
        fw = open(filepath, 'a')

    fw.write(str(day.hour)+":"+str(day.minute)+":"+str(day.second))
    fw.write("\t%.i\t%.2f\t%.2f\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t"%(line_num,emin,emax,ymax_mean,ymax_mean_e,sum_val,sum_val_e,back_val,back_val_e,net_val,net_val_e,fwhm,fwhm_e))
    fw.write(note+"\n")  
    fw.close()

    return ymax_mean,fwhm,y_back

def GetECPeakIntegral(ec,emin,emax):
    """
    @param ec       (ElementContainer)
    @param emin     (float)
    @param emax     (float)
    @retval result
    """
    xbin_min = 0
    xbin_max = 0
    sum_val = 0
    sum_val_e = 0
    sum_val_e2 = 0

    xbin = ec.PutX()
    ybin = ec.PutY()
    ebin = ec.PutE()

    if emin>emax:
        e_buf=emin
        emin=emax
        emax=e_buf

    for i in range(len(xbin)):
        if xbin[i] < emin:
                #xbin_min = i + 1
                xbin_min = i   # 2012/07/12 modify
        if xbin[i] < emax:
                #xbin_max = i + 2
                xbin_max = i   # 2012/07/12 modify

    for i in range(xbin_min,xbin_max):
        sum_val += ybin[i]
        sum_val_e2 += ebin[i]*ebin[i] 

    sum_val_e = math.sqrt(sum_val_e2)

    return sum_val,sum_val_e
